import io
from stripe._api_resource import APIResource
from stripe._custom_method import custom_method
from stripe._util import class_method_variant, sanitize_id


class TestCustomMethod(object):
    @custom_method("do_stuff", http_verb="post", http_path="do_the_thing")
    @custom_method(
        "do_list_stuff", http_verb="get", http_path="do_the_list_thing"
    )
    @custom_method(
        "do_stream_stuff",
        http_verb="post",
        http_path="do_the_stream_thing",
        is_streaming=True,
    )
    class MyResource(APIResource):
        OBJECT_NAME = "myresource"

        def do_stuff(self, idempotency_key=None, **params):
            url = self.instance_url() + "/do_the_thing"
            self._request_and_refresh(
                "post", url, {**params, "idempotency_key": idempotency_key}
            )
            return self

        def do_stream_stuff(self, idempotency_key=None, **params):
            url = self.instance_url() + "/do_the_stream_thing"
            return self._request_stream(
                "post", url, {**params, "idempotency_key": idempotency_key}
            )

        @classmethod
        def _cls_do_stuff_new_codegen(cls, id, **params):
            return cls._static_request(
                "post",
                "/v1/myresources/{id}/do_the_thing".format(id=sanitize_id(id)),
                params=params,
            )

        @class_method_variant("_cls_do_stuff_new_codegen")
        def do_stuff_new_codegen(self, **params):
            return self._request(
                "post",
                "/v1/myresources/{id}/do_the_thing".format(
                    id=sanitize_id(self.id)
                ),
                params=params,
            )

    def test_call_custom_method_class(self, http_client_mock):
        http_client_mock.stub_request(
            "post",
            path="/v1/myresources/mid/do_the_thing",
            rbody='{"id": "mid", "thing_done": true}',
            rheaders={"request-id": "req_id"},
        )

        obj = self.MyResource.do_stuff("mid", foo="bar")

        http_client_mock.assert_requested(
            "post",
            path="/v1/myresources/mid/do_the_thing",
            post_data="foo=bar",
        )
        assert obj.thing_done is True

    def test_call_custom_list_method_class_paginates(self, http_client_mock):
        http_client_mock.stub_request(
            "get",
            path="/v1/myresources/mid/do_the_list_thing",
            query_string="param1=abc&param2=def",
            rbody='{"object": "list", "url": "/v1/myresources/mid/do_the_list_thing", "has_more": true, "data": [{"id": "cus_1", "object": "customer"}, {"id": "cus_2", "object": "customer"}]}',
            rheaders={"request-id": "req_123"},
        )

        resp = self.MyResource.do_list_stuff("mid", param1="abc", param2="def")

        http_client_mock.assert_requested(
            "get",
            path="/v1/myresources/mid/do_the_list_thing",
            query_string="param1=abc&param2=def",
        )

        # Stub out the second request which will happen automatically.
        http_client_mock.stub_request(
            "get",
            path="/v1/myresources/mid/do_the_list_thing",
            query_string="param1=abc&param2=def&starting_after=cus_2",
            rbody='{"object": "list", "url": "/v1/myresources/mid/do_the_list_thing", "has_more": false, "data": [{"id": "cus_3", "object": "customer"}]}',
            rheaders={"request-id": "req_123"},
        )

        # Iterate through entire content
        ids = []
        for i in resp.auto_paging_iter():
            ids.append(i.id)

        # Explicitly assert that the pagination parameter were kept for the
        # second request along with the starting_after param.
        http_client_mock.assert_requested(
            "get",
            path="/v1/myresources/mid/do_the_list_thing",
            query_string="param1=abc&param2=def&starting_after=cus_2",
        )

        assert ids == ["cus_1", "cus_2", "cus_3"]

    def test_call_custom_stream_method_class(self, http_client_mock):
        http_client_mock.stub_request(
            "post",
            path="/v1/myresources/mid/do_the_stream_thing",
            rbody=io.BytesIO(str.encode("response body")),
            rheaders={"request-id": "req_id"},
        )

        resp = self.MyResource.do_stream_stuff("mid", foo="bar")

        http_client_mock.assert_requested(
            "post",
            path="/v1/myresources/mid/do_the_stream_thing",
            post_data="foo=bar",
        )

        body_content = resp.io.read()
        if hasattr(body_content, "decode"):
            body_content = body_content.decode("utf-8")

        assert body_content == "response body"

    def test_call_custom_method_class_with_object(self, http_client_mock):
        http_client_mock.stub_request(
            "post",
            path="/v1/myresources/mid/do_the_thing",
            rbody='{"id": "mid", "thing_done": true}',
            rheaders={"request-id": "req_id"},
        )

        obj = self.MyResource.construct_from({"id": "mid"}, "mykey")
        self.MyResource.do_stuff(obj, foo="bar")

        http_client_mock.assert_requested(
            "post",
            path="/v1/myresources/mid/do_the_thing",
            post_data="foo=bar",
        )
        assert obj.thing_done is True

    def test_call_custom_stream_method_class_with_object(
        self, http_client_mock
    ):
        http_client_mock.stub_request(
            "post",
            path="/v1/myresources/mid/do_the_stream_thing",
            rbody=io.BytesIO(str.encode("response body")),
            rheaders={"request-id": "req_id"},
        )

        obj = self.MyResource.construct_from({"id": "mid"}, "mykey")
        resp = self.MyResource.do_stream_stuff(obj, foo="bar")

        http_client_mock.assert_requested(
            "post",
            path="/v1/myresources/mid/do_the_stream_thing",
            post_data="foo=bar",
        )

        body_content = resp.io.read()
        if hasattr(body_content, "decode"):
            body_content = body_content.decode("utf-8")

        assert body_content == "response body"

    def test_call_custom_method_instance(self, http_client_mock):
        http_client_mock.stub_request(
            "post",
            path="/v1/myresources/mid/do_the_thing",
            rbody='{"id": "mid", "thing_done": true}',
            rheaders={"request-id": "req_id"},
        )

        obj = self.MyResource.construct_from({"id": "mid"}, "mykey")
        obj.do_stuff(foo="bar")

        http_client_mock.assert_requested(
            "post",
            path="/v1/myresources/mid/do_the_thing",
            post_data="foo=bar",
        )
        assert obj.thing_done is True

    def test_call_custom_stream_method_instance(self, http_client_mock):
        http_client_mock.stub_request(
            "post",
            path="/v1/myresources/mid/do_the_stream_thing",
            rbody=io.BytesIO(str.encode("response body")),
            rheaders={"request-id": "req_id"},
        )

        obj = self.MyResource.construct_from({"id": "mid"}, "mykey")
        resp = obj.do_stream_stuff(foo="bar")

        http_client_mock.assert_requested(
            "post",
            path="/v1/myresources/mid/do_the_stream_thing",
            post_data="foo=bar",
        )

        body_content = resp.io.read()
        if hasattr(body_content, "decode"):
            body_content = body_content.decode("utf-8")

        assert body_content == "response body"

    def test_call_custom_method_class_special_fields(self, http_client_mock):
        http_client_mock.stub_request(
            "post",
            path="/v1/myresources/mid/do_the_thing",
            rbody='{"id": "mid", "thing_done": true}',
            rheaders={"request-id": "req_id"},
        )

        self.MyResource.do_stuff(
            "mid",
            foo="bar",
            stripe_version="2017-08-15",
            api_key="APIKEY",
            idempotency_key="IdempotencyKey",
            stripe_account="Acc",
        )

        http_client_mock.assert_requested(
            "post",
            path="/v1/myresources/mid/do_the_thing",
            post_data="foo=bar",
            api_key="APIKEY",
            stripe_version="2017-08-15",
            stripe_account="Acc",
            idempotency_key="IdempotencyKey",
        )

    def test_call_custom_method_class_newcodegen_special_fields(
        self, http_client_mock
    ):
        http_client_mock.stub_request(
            "post",
            path="/v1/myresources/mid/do_the_thing",
            rbody='{"id": "mid", "thing_done": true}',
            rheaders={"request-id": "req_id"},
        )

        self.MyResource.do_stuff_new_codegen(
            "mid",
            foo="bar",
            stripe_version="2017-08-15",
            api_key="APIKEY",
            idempotency_key="IdempotencyKey",
            stripe_account="Acc",
        )

        http_client_mock.assert_requested(
            "post",
            path="/v1/myresources/mid/do_the_thing",
            post_data="foo=bar",
            api_key="APIKEY",
            stripe_version="2017-08-15",
            stripe_account="Acc",
            idempotency_key="IdempotencyKey",
        )

    def test_call_custom_method_instance_newcodegen_special_fields(
        self, http_client_mock
    ):
        http_client_mock.stub_request(
            "post",
            path="/v1/myresources/mid/do_the_thing",
            rbody='{"id": "mid", "thing_done": true}',
            rheaders={"request-id": "req_id"},
        )

        obj = self.MyResource.construct_from({"id": "mid"}, "mykey")
        obj.do_stuff_new_codegen(
            foo="bar",
            stripe_version="2017-08-15",
            api_key="APIKEY",
            idempotency_key="IdempotencyKey",
            stripe_account="Acc",
            headers={"extra_header": "val"},
        )

        http_client_mock.assert_requested(
            "post",
            path="/v1/myresources/mid/do_the_thing",
            post_data="foo=bar",
            api_key="APIKEY",
            stripe_version="2017-08-15",
            stripe_account="Acc",
            idempotency_key="IdempotencyKey",
            extra_headers={"extra_header": "val"},
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params.shared_payment._granted_token_create_params import (
        GrantedTokenCreateParams as GrantedTokenCreateParams,
        GrantedTokenCreateParamsUsageLimits as GrantedTokenCreateParamsUsageLimits,
    )
    from stripe.params.shared_payment._granted_token_retrieve_params import (
        GrantedTokenRetrieveParams as GrantedTokenRetrieveParams,
    )
    from stripe.params.shared_payment._granted_token_revoke_params import (
        GrantedTokenRevokeParams as GrantedTokenRevokeParams,
    )
    from stripe.params.shared_payment._issued_token_create_params import (
        IssuedTokenCreateParams as IssuedTokenCreateParams,
        IssuedTokenCreateParamsSellerDetails as IssuedTokenCreateParamsSellerDetails,
        IssuedTokenCreateParamsUsageLimits as IssuedTokenCreateParamsUsageLimits,
    )
    from stripe.params.shared_payment._issued_token_retrieve_params import (
        IssuedTokenRetrieveParams as IssuedTokenRetrieveParams,
    )
    from stripe.params.shared_payment._issued_token_revoke_params import (
        IssuedTokenRevokeParams as IssuedTokenRevokeParams,
    )

# name -> (import_target, is_submodule)
_import_map = {
    "GrantedTokenCreateParams": (
        "stripe.params.shared_payment._granted_token_create_params",
        False,
    ),
    "GrantedTokenCreateParamsUsageLimits": (
        "stripe.params.shared_payment._granted_token_create_params",
        False,
    ),
    "GrantedTokenRetrieveParams": (
        "stripe.params.shared_payment._granted_token_retrieve_params",
        False,
    ),
    "GrantedTokenRevokeParams": (
        "stripe.params.shared_payment._granted_token_revoke_params",
        False,
    ),
    "IssuedTokenCreateParams": (
        "stripe.params.shared_payment._issued_token_create_params",
        False,
    ),
    "IssuedTokenCreateParamsSellerDetails": (
        "stripe.params.shared_payment._issued_token_create_params",
        False,
    ),
    "IssuedTokenCreateParamsUsageLimits": (
        "stripe.params.shared_payment._issued_token_create_params",
        False,
    ),
    "IssuedTokenRetrieveParams": (
        "stripe.params.shared_payment._issued_token_retrieve_params",
        False,
    ),
    "IssuedTokenRevokeParams": (
        "stripe.params.shared_payment._issued_token_revoke_params",
        False,
    ),
}
if not TYPE_CHECKING:

    def __getattr__(name):
        try:
            target, is_submodule = _import_map[name]
            module = import_module(target)
            if is_submodule:
                return module

            return getattr(
                module,
                name,
            )
        except KeyError:
            raise AttributeError()



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params.v2.money_management import (
        financial_accounts as financial_accounts,
        test_helpers as test_helpers,
    )
    from stripe.params.v2.money_management._adjustment_list_params import (
        AdjustmentListParams as AdjustmentListParams,
    )
    from stripe.params.v2.money_management._adjustment_retrieve_params import (
        AdjustmentRetrieveParams as AdjustmentRetrieveParams,
    )
    from stripe.params.v2.money_management._currency_conversion_create_params import (
        CurrencyConversionCreateParams as CurrencyConversionCreateParams,
        CurrencyConversionCreateParamsFrom as CurrencyConversionCreateParamsFrom,
        CurrencyConversionCreateParamsTo as CurrencyConversionCreateParamsTo,
    )
    from stripe.params.v2.money_management._currency_conversion_list_params import (
        CurrencyConversionListParams as CurrencyConversionListParams,
    )
    from stripe.params.v2.money_management._currency_conversion_retrieve_params import (
        CurrencyConversionRetrieveParams as CurrencyConversionRetrieveParams,
    )
    from stripe.params.v2.money_management._debit_dispute_create_params import (
        DebitDisputeCreateParams as DebitDisputeCreateParams,
        DebitDisputeCreateParamsBankTransfer as DebitDisputeCreateParamsBankTransfer,
    )
    from stripe.params.v2.money_management._debit_dispute_list_params import (
        DebitDisputeListParams as DebitDisputeListParams,
    )
    from stripe.params.v2.money_management._debit_dispute_retrieve_params import (
        DebitDisputeRetrieveParams as DebitDisputeRetrieveParams,
    )
    from stripe.params.v2.money_management._financial_account_close_params import (
        FinancialAccountCloseParams as FinancialAccountCloseParams,
        FinancialAccountCloseParamsForwardingSettings as FinancialAccountCloseParamsForwardingSettings,
    )
    from stripe.params.v2.money_management._financial_account_create_params import (
        FinancialAccountCreateParams as FinancialAccountCreateParams,
        FinancialAccountCreateParamsStorage as FinancialAccountCreateParamsStorage,
    )
    from stripe.params.v2.money_management._financial_account_list_params import (
        FinancialAccountListParams as FinancialAccountListParams,
    )
    from stripe.params.v2.money_management._financial_account_retrieve_params import (
        FinancialAccountRetrieveParams as FinancialAccountRetrieveParams,
    )
    from stripe.params.v2.money_management._financial_account_update_params import (
        FinancialAccountUpdateParams as FinancialAccountUpdateParams,
        FinancialAccountUpdateParamsStorage as FinancialAccountUpdateParamsStorage,
    )
    from stripe.params.v2.money_management._financial_address_create_params import (
        FinancialAddressCreateParams as FinancialAddressCreateParams,
        FinancialAddressCreateParamsCryptoProperties as FinancialAddressCreateParamsCryptoProperties,
        FinancialAddressCreateParamsSepaBankAccount as FinancialAddressCreateParamsSepaBankAccount,
    )
    from stripe.params.v2.money_management._financial_address_list_params import (
        FinancialAddressListParams as FinancialAddressListParams,
    )
    from stripe.params.v2.money_management._financial_address_retrieve_params import (
        FinancialAddressRetrieveParams as FinancialAddressRetrieveParams,
    )
    from stripe.params.v2.money_management._inbound_transfer_create_params import (
        InboundTransferCreateParams as InboundTransferCreateParams,
        InboundTransferCreateParamsFrom as InboundTransferCreateParamsFrom,
        InboundTransferCreateParamsTo as InboundTransferCreateParamsTo,
    )
    from stripe.params.v2.money_management._inbound_transfer_list_params import (
        InboundTransferListParams as InboundTransferListParams,
    )
    from stripe.params.v2.money_management._inbound_transfer_retrieve_params import (
        InboundTransferRetrieveParams as InboundTransferRetrieveParams,
    )
    from stripe.params.v2.money_management._outbound_payment_cancel_params import (
        OutboundPaymentCancelParams as OutboundPaymentCancelParams,
    )
    from stripe.params.v2.money_management._outbound_payment_create_params import (
        OutboundPaymentCreateParams as OutboundPaymentCreateParams,
        OutboundPaymentCreateParamsDeliveryOptions as OutboundPaymentCreateParamsDeliveryOptions,
        OutboundPaymentCreateParamsDeliveryOptionsPaperCheck as OutboundPaymentCreateParamsDeliveryOptionsPaperCheck,
        OutboundPaymentCreateParamsFrom as OutboundPaymentCreateParamsFrom,
        OutboundPaymentCreateParamsRecipientNotification as OutboundPaymentCreateParamsRecipientNotification,
        OutboundPaymentCreateParamsTo as OutboundPaymentCreateParamsTo,
        OutboundPaymentCreateParamsToPayoutMethodOptions as OutboundPaymentCreateParamsToPayoutMethodOptions,
        OutboundPaymentCreateParamsToPayoutMethodOptionsBankAccount as OutboundPaymentCreateParamsToPayoutMethodOptionsBankAccount,
        OutboundPaymentCreateParamsToPayoutMethodOptionsBankAccountPreferredNetworkOptions as OutboundPaymentCreateParamsToPayoutMethodOptionsBankAccountPreferredNetworkOptions,
        OutboundPaymentCreateParamsToPayoutMethodOptionsBankAccountPreferredNetworkOptionsAch as OutboundPaymentCreateParamsToPayoutMethodOptionsBankAccountPreferredNetworkOptionsAch,
    )
    from stripe.params.v2.money_management._outbound_payment_list_params import (
        OutboundPaymentListParams as OutboundPaymentListParams,
    )
    from stripe.params.v2.money_management._outbound_payment_quote_create_params import (
        OutboundPaymentQuoteCreateParams as OutboundPaymentQuoteCreateParams,
        OutboundPaymentQuoteCreateParamsDeliveryOptions as OutboundPaymentQuoteCreateParamsDeliveryOptions,
        OutboundPaymentQuoteCreateParamsFrom as OutboundPaymentQuoteCreateParamsFrom,
        OutboundPaymentQuoteCreateParamsTo as OutboundPaymentQuoteCreateParamsTo,
    )
    from stripe.params.v2.money_management._outbound_payment_quote_retrieve_params import (
        OutboundPaymentQuoteRetrieveParams as OutboundPaymentQuoteRetrieveParams,
    )
    from stripe.params.v2.money_management._outbound_payment_retrieve_params import (
        OutboundPaymentRetrieveParams as OutboundPaymentRetrieveParams,
    )
    from stripe.params.v2.money_management._outbound_setup_intent_cancel_params import (
        OutboundSetupIntentCancelParams as OutboundSetupIntentCancelParams,
    )
    from stripe.params.v2.money_management._outbound_setup_intent_create_params import (
        OutboundSetupIntentCreateParams as OutboundSetupIntentCreateParams,
        OutboundSetupIntentCreateParamsPayoutMethodData as OutboundSetupIntentCreateParamsPayoutMethodData,
        OutboundSetupIntentCreateParamsPayoutMethodDataBankAccount as OutboundSetupIntentCreateParamsPayoutMethodDataBankAccount,
        OutboundSetupIntentCreateParamsPayoutMethodDataCard as OutboundSetupIntentCreateParamsPayoutMethodDataCard,
        OutboundSetupIntentCreateParamsPayoutMethodDataCryptoWallet as OutboundSetupIntentCreateParamsPayoutMethodDataCryptoWallet,
    )
    from stripe.params.v2.money_management._outbound_setup_intent_list_params import (
        OutboundSetupIntentListParams as OutboundSetupIntentListParams,
    )
    from stripe.params.v2.money_management._outbound_setup_intent_retrieve_params import (
        OutboundSetupIntentRetrieveParams as OutboundSetupIntentRetrieveParams,
    )
    from stripe.params.v2.money_management._outbound_setup_intent_update_params import (
        OutboundSetupIntentUpdateParams as OutboundSetupIntentUpdateParams,
        OutboundSetupIntentUpdateParamsPayoutMethodData as OutboundSetupIntentUpdateParamsPayoutMethodData,
        OutboundSetupIntentUpdateParamsPayoutMethodDataBankAccount as OutboundSetupIntentUpdateParamsPayoutMethodDataBankAccount,
        OutboundSetupIntentUpdateParamsPayoutMethodDataCard as OutboundSetupIntentUpdateParamsPayoutMethodDataCard,
    )
    from stripe.params.v2.money_management._outbound_transfer_cancel_params import (
        OutboundTransferCancelParams as OutboundTransferCancelParams,
    )
    from stripe.params.v2.money_management._outbound_transfer_create_params import (
        OutboundTransferCreateParams as OutboundTransferCreateParams,
        OutboundTransferCreateParamsDeliveryOptions as OutboundTransferCreateParamsDeliveryOptions,
        OutboundTransferCreateParamsFrom as OutboundTransferCreateParamsFrom,
        OutboundTransferCreateParamsTo as OutboundTransferCreateParamsTo,
        OutboundTransferCreateParamsToPayoutMethodOptions as OutboundTransferCreateParamsToPayoutMethodOptions,
        OutboundTransferCreateParamsToPayoutMethodOptionsBankAccount as OutboundTransferCreateParamsToPayoutMethodOptionsBankAccount,
    )
    from stripe.params.v2.money_management._outbound_transfer_list_params import (
        OutboundTransferListParams as OutboundTransferListParams,
    )
    from stripe.params.v2.money_management._outbound_transfer_retrieve_params import (
        OutboundTransferRetrieveParams as OutboundTransferRetrieveParams,
    )
    from stripe.params.v2.money_management._payout_intent_cancel_params import (
        PayoutIntentCancelParams as PayoutIntentCancelParams,
    )
    from stripe.params.v2.money_management._payout_intent_create_params import (
        PayoutIntentCreateParams as PayoutIntentCreateParams,
        PayoutIntentCreateParamsFrom as PayoutIntentCreateParamsFrom,
        PayoutIntentCreateParamsRecipientNotification as PayoutIntentCreateParamsRecipientNotification,
        PayoutIntentCreateParamsScheduleOptions as PayoutIntentCreateParamsScheduleOptions,
        PayoutIntentCreateParamsTo as PayoutIntentCreateParamsTo,
        PayoutIntentCreateParamsToPayoutMethodOptions as PayoutIntentCreateParamsToPayoutMethodOptions,
        PayoutIntentCreateParamsToPayoutMethodOptionsBankAccount as PayoutIntentCreateParamsToPayoutMethodOptionsBankAccount,
        PayoutIntentCreateParamsToPayoutMethodOptionsBankAccountPreferredNetworkOptions as PayoutIntentCreateParamsToPayoutMethodOptionsBankAccountPreferredNetworkOptions,
        PayoutIntentCreateParamsToPayoutMethodOptionsBankAccountPreferredNetworkOptionsAch as PayoutIntentCreateParamsToPayoutMethodOptionsBankAccountPreferredNetworkOptionsAch,
    )
    from stripe.params.v2.money_management._payout_intent_list_params import (
        PayoutIntentListParams as PayoutIntentListParams,
    )
    from stripe.params.v2.money_management._payout_intent_retrieve_params import (
        PayoutIntentRetrieveParams as PayoutIntentRetrieveParams,
    )
    from stripe.params.v2.money_management._payout_intent_update_params import (
        PayoutIntentUpdateParams as PayoutIntentUpdateParams,
        PayoutIntentUpdateParamsFrom as PayoutIntentUpdateParamsFrom,
        PayoutIntentUpdateParamsRecipientNotification as PayoutIntentUpdateParamsRecipientNotification,
        PayoutIntentUpdateParamsScheduleOptions as PayoutIntentUpdateParamsScheduleOptions,
        PayoutIntentUpdateParamsTo as PayoutIntentUpdateParamsTo,
        PayoutIntentUpdateParamsToPayoutMethodOptions as PayoutIntentUpdateParamsToPayoutMethodOptions,
        PayoutIntentUpdateParamsToPayoutMethodOptionsBankAccount as PayoutIntentUpdateParamsToPayoutMethodOptionsBankAccount,
        PayoutIntentUpdateParamsToPayoutMethodOptionsBankAccountPreferredNetworkOptions as PayoutIntentUpdateParamsToPayoutMethodOptionsBankAccountPreferredNetworkOptions,
        PayoutIntentUpdateParamsToPayoutMethodOptionsBankAccountPreferredNetworkOptionsAch as PayoutIntentUpdateParamsToPayoutMethodOptionsBankAccountPreferredNetworkOptionsAch,
    )
    from stripe.params.v2.money_management._payout_method_archive_params import (
        PayoutMethodArchiveParams as PayoutMethodArchiveParams,
    )
    from stripe.params.v2.money_management._payout_method_list_params import (
        PayoutMethodListParams as PayoutMethodListParams,
        PayoutMethodListParamsUsageStatus as PayoutMethodListParamsUsageStatus,
    )
    from stripe.params.v2.money_management._payout_method_retrieve_params import (
        PayoutMethodRetrieveParams as PayoutMethodRetrieveParams,
    )
    from stripe.params.v2.money_management._payout_method_unarchive_params import (
        PayoutMethodUnarchiveParams as PayoutMethodUnarchiveParams,
    )
    from stripe.params.v2.money_management._payout_methods_bank_account_spec_retrieve_params import (
        PayoutMethodsBankAccountSpecRetrieveParams as PayoutMethodsBankAccountSpecRetrieveParams,
    )
    from stripe.params.v2.money_management._received_credit_list_params import (
        ReceivedCreditListParams as ReceivedCreditListParams,
    )
    from stripe.params.v2.money_management._received_credit_retrieve_params import (
        ReceivedCreditRetrieveParams as ReceivedCreditRetrieveParams,
    )
    from stripe.params.v2.money_management._received_debit_list_params import (
        ReceivedDebitListParams as ReceivedDebitListParams,
    )
    from stripe.params.v2.money_management._received_debit_retrieve_params import (
        ReceivedDebitRetrieveParams as ReceivedDebitRetrieveParams,
    )
    from stripe.params.v2.money_management._recipient_verification_acknowledge_params import (
        RecipientVerificationAcknowledgeParams as RecipientVerificationAcknowledgeParams,
    )
    from stripe.params.v2.money_management._recipient_verification_create_params import (
        RecipientVerificationCreateParams as RecipientVerificationCreateParams,
    )
    from stripe.params.v2.money_management._recipient_verification_retrieve_params import (
        RecipientVerificationRetrieveParams as RecipientVerificationRetrieveParams,
    )
    from stripe.params.v2.money_management._transaction_entry_list_params import (
        TransactionEntryListParams as TransactionEntryListParams,
    )
    from stripe.params.v2.money_management._transaction_entry_retrieve_params import (
        TransactionEntryRetrieveParams as TransactionEntryRetrieveParams,
    )
    from stripe.params.v2.money_management._transaction_list_params import (
        TransactionListParams as TransactionListParams,
    )
    from stripe.params.v2.money_management._transaction_retrieve_params import (
        TransactionRetrieveParams as TransactionRetrieveParams,
    )

# name -> (import_target, is_submodule)
_import_map = {
    "financial_accounts": (
        "stripe.params.v2.money_management.financial_accounts",
        True,
    ),
    "test_helpers": ("stripe.params.v2.money_management.test_helpers", True),
    "AdjustmentListParams": (
        "stripe.params.v2.money_management._adjustment_list_params",
        False,
    ),
    "AdjustmentRetrieveParams": (
        "stripe.params.v2.money_management._adjustment_retrieve_params",
        False,
    ),
    "CurrencyConversionCreateParams": (
        "stripe.params.v2.money_management._currency_conversion_create_params",
        False,
    ),
    "CurrencyConversionCreateParamsFrom": (
        "stripe.params.v2.money_management._currency_conversion_create_params",
        False,
    ),
    "CurrencyConversionCreateParamsTo": (
        "stripe.params.v2.money_management._currency_conversion_create_params",
        False,
    ),
    "CurrencyConversionListParams": (
        "stripe.params.v2.money_management._currency_conversion_list_params",
        False,
    ),
    "CurrencyConversionRetrieveParams": (
        "stripe.params.v2.money_management._currency_conversion_retrieve_params",
        False,
    ),
    "DebitDisputeCreateParams": (
        "stripe.params.v2.money_management._debit_dispute_create_params",
        False,
    ),
    "DebitDisputeCreateParamsBankTransfer": (
        "stripe.params.v2.money_management._debit_dispute_create_params",
        False,
    ),
    "DebitDisputeListParams": (
        "stripe.params.v2.money_management._debit_dispute_list_params",
        False,
    ),
    "DebitDisputeRetrieveParams": (
        "stripe.params.v2.money_management._debit_dispute_retrieve_params",
        False,
    ),
    "FinancialAccountCloseParams": (
        "stripe.params.v2.money_management._financial_account_close_params",
        False,
    ),
    "FinancialAccountCloseParamsForwardingSettings": (
        "stripe.params.v2.money_management._financial_account_close_params",
        False,
    ),
    "FinancialAccountCreateParams": (
        "stripe.params.v2.money_management._financial_account_create_params",
        False,
    ),
    "FinancialAccountCreateParamsStorage": (
        "stripe.params.v2.money_management._financial_account_create_params",
        False,
    ),
    "FinancialAccountListParams": (
        "stripe.params.v2.money_management._financial_account_list_params",
        False,
    ),
    "FinancialAccountRetrieveParams": (
        "stripe.params.v2.money_management._financial_account_retrieve_params",
        False,
    ),
    "FinancialAccountUpdateParams": (
        "stripe.params.v2.money_management._financial_account_update_params",
        False,
    ),
    "FinancialAccountUpdateParamsStorage": (
        "stripe.params.v2.money_management._financial_account_update_params",
        False,
    ),
    "FinancialAddressCreateParams": (
        "stripe.params.v2.money_management._financial_address_create_params",
        False,
    ),
    "FinancialAddressCreateParamsCryptoProperties": (
        "stripe.params.v2.money_management._financial_address_create_params",
        False,
    ),
    "FinancialAddressCreateParamsSepaBankAccount": (
        "stripe.params.v2.money_management._financial_address_create_params",
        False,
    ),
    "FinancialAddressListParams": (
        "stripe.params.v2.money_management._financial_address_list_params",
        False,
    ),
    "FinancialAddressRetrieveParams": (
        "stripe.params.v2.money_management._financial_address_retrieve_params",
        False,
    ),
    "InboundTransferCreateParams": (
        "stripe.params.v2.money_management._inbound_transfer_create_params",
        False,
    ),
    "InboundTransferCreateParamsFrom": (
        "stripe.params.v2.money_management._inbound_transfer_create_params",
        False,
    ),
    "InboundTransferCreateParamsTo": (
        "stripe.params.v2.money_management._inbound_transfer_create_params",
        False,
    ),
    "InboundTransferListParams": (
        "stripe.params.v2.money_management._inbound_transfer_list_params",
        False,
    ),
    "InboundTransferRetrieveParams": (
        "stripe.params.v2.money_management._inbound_transfer_retrieve_params",
        False,
    ),
    "OutboundPaymentCancelParams": (
        "stripe.params.v2.money_management._outbound_payment_cancel_params",
        False,
    ),
    "OutboundPaymentCreateParams": (
        "stripe.params.v2.money_management._outbound_payment_create_params",
        False,
    ),
    "OutboundPaymentCreateParamsDeliveryOptions": (
        "stripe.params.v2.money_management._outbound_payment_create_params",
        False,
    ),
    "OutboundPaymentCreateParamsDeliveryOptionsPaperCheck": (
        "stripe.params.v2.money_management._outbound_payment_create_params",
        False,
    ),
    "OutboundPaymentCreateParamsFrom": (
        "stripe.params.v2.money_management._outbound_payment_create_params",
        False,
    ),
    "OutboundPaymentCreateParamsRecipientNotification": (
        "stripe.params.v2.money_management._outbound_payment_create_params",
        False,
    ),
    "OutboundPaymentCreateParamsTo": (
        "stripe.params.v2.money_management._outbound_payment_create_params",
        False,
    ),
    "OutboundPaymentCreateParamsToPayoutMethodOptions": (
        "stripe.params.v2.money_management._outbound_payment_create_params",
        False,
    ),
    "OutboundPaymentCreateParamsToPayoutMethodOptionsBankAccount": (
        "stripe.params.v2.money_management._outbound_payment_create_params",
        False,
    ),
    "OutboundPaymentCreateParamsToPayoutMethodOptionsBankAccountPreferredNetworkOptions": (
        "stripe.params.v2.money_management._outbound_payment_create_params",
        False,
    ),
    "OutboundPaymentCreateParamsToPayoutMethodOptionsBankAccountPreferredNetworkOptionsAch": (
        "stripe.params.v2.money_management._outbound_payment_create_params",
        False,
    ),
    "OutboundPaymentListParams": (
        "stripe.params.v2.money_management._outbound_payment_list_params",
        False,
    ),
    "OutboundPaymentQuoteCreateParams": (
        "stripe.params.v2.money_management._outbound_payment_quote_create_params",
        False,
    ),
    "OutboundPaymentQuoteCreateParamsDeliveryOptions": (
        "stripe.params.v2.money_management._outbound_payment_quote_create_params",
        False,
    ),
    "OutboundPaymentQuoteCreateParamsFrom": (
        "stripe.params.v2.money_management._outbound_payment_quote_create_params",
        False,
    ),
    "OutboundPaymentQuoteCreateParamsTo": (
        "stripe.params.v2.money_management._outbound_payment_quote_create_params",
        False,
    ),
    "OutboundPaymentQuoteRetrieveParams": (
        "stripe.params.v2.money_management._outbound_payment_quote_retrieve_params",
        False,
    ),
    "OutboundPaymentRetrieveParams": (
        "stripe.params.v2.money_management._outbound_payment_retrieve_params",
        False,
    ),
    "OutboundSetupIntentCancelParams": (
        "stripe.params.v2.money_management._outbound_setup_intent_cancel_params",
        False,
    ),
    "OutboundSetupIntentCreateParams": (
        "stripe.params.v2.money_management._outbound_setup_intent_create_params",
        False,
    ),
    "OutboundSetupIntentCreateParamsPayoutMethodData": (
        "stripe.params.v2.money_management._outbound_setup_intent_create_params",
        False,
    ),
    "OutboundSetupIntentCreateParamsPayoutMethodDataBankAccount": (
        "stripe.params.v2.money_management._outbound_setup_intent_create_params",
        False,
    ),
    "OutboundSetupIntentCreateParamsPayoutMethodDataCard": (
        "stripe.params.v2.money_management._outbound_setup_intent_create_params",
        False,
    ),
    "OutboundSetupIntentCreateParamsPayoutMethodDataCryptoWallet": (
        "stripe.params.v2.money_management._outbound_setup_intent_create_params",
        False,
    ),
    "OutboundSetupIntentListParams": (
        "stripe.params.v2.money_management._outbound_setup_intent_list_params",
        False,
    ),
    "OutboundSetupIntentRetrieveParams": (
        "stripe.params.v2.money_management._outbound_setup_intent_retrieve_params",
        False,
    ),
    "OutboundSetupIntentUpdateParams": (
        "stripe.params.v2.money_management._outbound_setup_intent_update_params",
        False,
    ),
    "OutboundSetupIntentUpdateParamsPayoutMethodData": (
        "stripe.params.v2.money_management._outbound_setup_intent_update_params",
        False,
    ),
    "OutboundSetupIntentUpdateParamsPayoutMethodDataBankAccount": (
        "stripe.params.v2.money_management._outbound_setup_intent_update_params",
        False,
    ),
    "OutboundSetupIntentUpdateParamsPayoutMethodDataCard": (
        "stripe.params.v2.money_management._outbound_setup_intent_update_params",
        False,
    ),
    "OutboundTransferCancelParams": (
        "stripe.params.v2.money_management._outbound_transfer_cancel_params",
        False,
    ),
    "OutboundTransferCreateParams": (
        "stripe.params.v2.money_management._outbound_transfer_create_params",
        False,
    ),
    "OutboundTransferCreateParamsDeliveryOptions": (
        "stripe.params.v2.money_management._outbound_transfer_create_params",
        False,
    ),
    "OutboundTransferCreateParamsFrom": (
        "stripe.params.v2.money_management._outbound_transfer_create_params",
        False,
    ),
    "OutboundTransferCreateParamsTo": (
        "stripe.params.v2.money_management._outbound_transfer_create_params",
        False,
    ),
    "OutboundTransferCreateParamsToPayoutMethodOptions": (
        "stripe.params.v2.money_management._outbound_transfer_create_params",
        False,
    ),
    "OutboundTransferCreateParamsToPayoutMethodOptionsBankAccount": (
        "stripe.params.v2.money_management._outbound_transfer_create_params",
        False,
    ),
    "OutboundTransferListParams": (
        "stripe.params.v2.money_management._outbound_transfer_list_params",
        False,
    ),
    "OutboundTransferRetrieveParams": (
        "stripe.params.v2.money_management._outbound_transfer_retrieve_params",
        False,
    ),
    "PayoutIntentCancelParams": (
        "stripe.params.v2.money_management._payout_intent_cancel_params",
        False,
    ),
    "PayoutIntentCreateParams": (
        "stripe.params.v2.money_management._payout_intent_create_params",
        False,
    ),
    "PayoutIntentCreateParamsFrom": (
        "stripe.params.v2.money_management._payout_intent_create_params",
        False,
    ),
    "PayoutIntentCreateParamsRecipientNotification": (
        "stripe.params.v2.money_management._payout_intent_create_params",
        False,
    ),
    "PayoutIntentCreateParamsScheduleOptions": (
        "stripe.params.v2.money_management._payout_intent_create_params",
        False,
    ),
    "PayoutIntentCreateParamsTo": (
        "stripe.params.v2.money_management._payout_intent_create_params",
        False,
    ),
    "PayoutIntentCreateParamsToPayoutMethodOptions": (
        "stripe.params.v2.money_management._payout_intent_create_params",
        False,
    ),
    "PayoutIntentCreateParamsToPayoutMethodOptionsBankAccount": (
        "stripe.params.v2.money_management._payout_intent_create_params",
        False,
    ),
    "PayoutIntentCreateParamsToPayoutMethodOptionsBankAccountPreferredNetworkOptions": (
        "stripe.params.v2.money_management._payout_intent_create_params",
        False,
    ),
    "PayoutIntentCreateParamsToPayoutMethodOptionsBankAccountPreferredNetworkOptionsAch": (
        "stripe.params.v2.money_management._payout_intent_create_params",
        False,
    ),
    "PayoutIntentListParams": (
        "stripe.params.v2.money_management._payout_intent_list_params",
        False,
    ),
    "PayoutIntentRetrieveParams": (
        "stripe.params.v2.money_management._payout_intent_retrieve_params",
        False,
    ),
    "PayoutIntentUpdateParams": (
        "stripe.params.v2.money_management._payout_intent_update_params",
        False,
    ),
    "PayoutIntentUpdateParamsFrom": (
        "stripe.params.v2.money_management._payout_intent_update_params",
        False,
    ),
    "PayoutIntentUpdateParamsRecipientNotification": (
        "stripe.params.v2.money_management._payout_intent_update_params",
        False,
    ),
    "PayoutIntentUpdateParamsScheduleOptions": (
        "stripe.params.v2.money_management._payout_intent_update_params",
        False,
    ),
    "PayoutIntentUpdateParamsTo": (
        "stripe.params.v2.money_management._payout_intent_update_params",
        False,
    ),
    "PayoutIntentUpdateParamsToPayoutMethodOptions": (
        "stripe.params.v2.money_management._payout_intent_update_params",
        False,
    ),
    "PayoutIntentUpdateParamsToPayoutMethodOptionsBankAccount": (
        "stripe.params.v2.money_management._payout_intent_update_params",
        False,
    ),
    "PayoutIntentUpdateParamsToPayoutMethodOptionsBankAccountPreferredNetworkOptions": (
        "stripe.params.v2.money_management._payout_intent_update_params",
        False,
    ),
    "PayoutIntentUpdateParamsToPayoutMethodOptionsBankAccountPreferredNetworkOptionsAch": (
        "stripe.params.v2.money_management._payout_intent_update_params",
        False,
    ),
    "PayoutMethodArchiveParams": (
        "stripe.params.v2.money_management._payout_method_archive_params",
        False,
    ),
    "PayoutMethodListParams": (
        "stripe.params.v2.money_management._payout_method_list_params",
        False,
    ),
    "PayoutMethodListParamsUsageStatus": (
        "stripe.params.v2.money_management._payout_method_list_params",
        False,
    ),
    "PayoutMethodRetrieveParams": (
        "stripe.params.v2.money_management._payout_method_retrieve_params",
        False,
    ),
    "PayoutMethodUnarchiveParams": (
        "stripe.params.v2.money_management._payout_method_unarchive_params",
        False,
    ),
    "PayoutMethodsBankAccountSpecRetrieveParams": (
        "stripe.params.v2.money_management._payout_methods_bank_account_spec_retrieve_params",
        False,
    ),
    "ReceivedCreditListParams": (
        "stripe.params.v2.money_management._received_credit_list_params",
        False,
    ),
    "ReceivedCreditRetrieveParams": (
        "stripe.params.v2.money_management._received_credit_retrieve_params",
        False,
    ),
    "ReceivedDebitListParams": (
        "stripe.params.v2.money_management._received_debit_list_params",
        False,
    ),
    "ReceivedDebitRetrieveParams": (
        "stripe.params.v2.money_management._received_debit_retrieve_params",
        False,
    ),
    "RecipientVerificationAcknowledgeParams": (
        "stripe.params.v2.money_management._recipient_verification_acknowledge_params",
        False,
    ),
    "RecipientVerificationCreateParams": (
        "stripe.params.v2.money_management._recipient_verification_create_params",
        False,
    ),
    "RecipientVerificationRetrieveParams": (
        "stripe.params.v2.money_management._recipient_verification_retrieve_params",
        False,
    ),
    "TransactionEntryListParams": (
        "stripe.params.v2.money_management._transaction_entry_list_params",
        False,
    ),
    "TransactionEntryRetrieveParams": (
        "stripe.params.v2.money_management._transaction_entry_retrieve_params",
        False,
    ),
    "TransactionListParams": (
        "stripe.params.v2.money_management._transaction_list_params",
        False,
    ),
    "TransactionRetrieveParams": (
        "stripe.params.v2.money_management._transaction_retrieve_params",
        False,
    ),
}
if not TYPE_CHECKING:

    def __getattr__(name):
        try:
            target, is_submodule = _import_map[name]
            module = import_module(target)
            if is_submodule:
                return module

            return getattr(
                module,
                name,
            )
        except KeyError:
            raise AttributeError()



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from decimal import Decimal
from stripe._stripe_object import UntypedStripeObject
from typing import Dict, List
from typing_extensions import Literal, NotRequired, TypedDict


class PersonCreateParams(TypedDict):
    additional_addresses: NotRequired[
        List["PersonCreateParamsAdditionalAddress"]
    ]
    """
    Additional addresses associated with the person.
    """
    additional_names: NotRequired[List["PersonCreateParamsAdditionalName"]]
    """
    Additional names (e.g. aliases) associated with the person.
    """
    additional_terms_of_service: NotRequired[
        "PersonCreateParamsAdditionalTermsOfService"
    ]
    """
    Attestations of accepted terms of service agreements.
    """
    address: NotRequired["PersonCreateParamsAddress"]
    """
    The person's residential address.
    """
    date_of_birth: NotRequired["PersonCreateParamsDateOfBirth"]
    """
    The person's date of birth.
    """
    documents: NotRequired["PersonCreateParamsDocuments"]
    """
    Documents that may be submitted to satisfy various informational requests.
    """
    email: NotRequired[str]
    """
    Email.
    """
    given_name: NotRequired[str]
    """
    The person's first name.
    """
    id_numbers: NotRequired[List["PersonCreateParamsIdNumber"]]
    """
    The identification numbers (e.g., SSN) associated with the person.
    """
    legal_gender: NotRequired[Literal["female", "male"]]
    """
    The person's gender (International regulations require either "male" or "female").
    """
    metadata: NotRequired["Dict[str, str]|UntypedStripeObject[str]"]
    """
    Set of key-value pairs that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    nationalities: NotRequired[List[str]]
    """
    The nationalities (countries) this person is associated with.
    """
    person_token: NotRequired[str]
    """
    The person token generated by the person token api.
    """
    phone: NotRequired[str]
    """
    The phone number for this person.
    """
    political_exposure: NotRequired[Literal["existing", "none"]]
    """
    The person's political exposure.
    """
    relationship: NotRequired["PersonCreateParamsRelationship"]
    """
    The relationship that this person has with the Account's business or legal entity.
    """
    script_addresses: NotRequired["PersonCreateParamsScriptAddresses"]
    """
    The script addresses (e.g., non-Latin characters) associated with the person.
    """
    script_names: NotRequired["PersonCreateParamsScriptNames"]
    """
    The script names (e.g. non-Latin characters) associated with the person.
    """
    surname: NotRequired[str]
    """
    The person's last name.
    """


class PersonCreateParamsAdditionalAddress(TypedDict):
    city: NotRequired[str]
    """
    City, district, suburb, town, or village.
    """
    country: str
    """
    Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
    """
    line1: NotRequired[str]
    """
    Address line 1 (e.g., street, PO Box, or company name).
    """
    line2: NotRequired[str]
    """
    Address line 2 (e.g., apartment, suite, unit, or building).
    """
    postal_code: NotRequired[str]
    """
    ZIP or postal code.
    """
    purpose: Literal["registered"]
    """
    Purpose of additional address.
    """
    state: NotRequired[str]
    """
    State, county, province, or region.
    """
    town: NotRequired[str]
    """
    Town or district.
    """


class PersonCreateParamsAdditionalName(TypedDict):
    full_name: NotRequired[str]
    """
    The person's full name.
    """
    given_name: NotRequired[str]
    """
    The person's first or given name.
    """
    purpose: Literal["alias", "maiden"]
    """
    The purpose or type of the additional name.
    """
    surname: NotRequired[str]
    """
    The person's last or family name.
    """


class PersonCreateParamsAdditionalTermsOfService(TypedDict):
    account: NotRequired["PersonCreateParamsAdditionalTermsOfServiceAccount"]
    """
    Stripe terms of service agreement.
    """


class PersonCreateParamsAdditionalTermsOfServiceAccount(TypedDict):
    date: str
    """
    The time when the Account's representative accepted the terms of service. Represented as a RFC 3339 date & time UTC value in millisecond precision, for example: 2022-09-18T13:22:18.123Z.
    """
    ip: str
    """
    The IP address from which the Account's representative accepted the terms of service.
    """
    user_agent: NotRequired[str]
    """
    The user agent of the browser from which the Account's representative accepted the terms of service.
    """


class PersonCreateParamsAddress(TypedDict):
    city: NotRequired[str]
    """
    City, district, suburb, town, or village.
    """
    country: str
    """
    Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
    """
    line1: NotRequired[str]
    """
    Address line 1 (e.g., street, PO Box, or company name).
    """
    line2: NotRequired[str]
    """
    Address line 2 (e.g., apartment, suite, unit, or building).
    """
    postal_code: NotRequired[str]
    """
    ZIP or postal code.
    """
    state: NotRequired[str]
    """
    State, county, province, or region.
    """
    town: NotRequired[str]
    """
    Town or district.
    """


class PersonCreateParamsDateOfBirth(TypedDict):
    day: int
    """
    The day of birth.
    """
    month: int
    """
    The month of birth.
    """
    year: int
    """
    The year of birth.
    """


class PersonCreateParamsDocuments(TypedDict):
    company_authorization: NotRequired[
        "PersonCreateParamsDocumentsCompanyAuthorization"
    ]
    """
    One or more documents that demonstrate proof that this person is authorized to represent the company.
    """
    passport: NotRequired["PersonCreateParamsDocumentsPassport"]
    """
    One or more documents showing the person's passport page with photo and personal data.
    """
    primary_verification: NotRequired[
        "PersonCreateParamsDocumentsPrimaryVerification"
    ]
    """
    An identifying document showing the person's name, either a passport or local ID card.
    """
    secondary_verification: NotRequired[
        "PersonCreateParamsDocumentsSecondaryVerification"
    ]
    """
    A document showing address, either a passport, local ID card, or utility bill from a well-known utility company.
    """
    visa: NotRequired["PersonCreateParamsDocumentsVisa"]
    """
    One or more documents showing the person's visa required for living in the country where they are residing.
    """


class PersonCreateParamsDocumentsCompanyAuthorization(TypedDict):
    files: List[str]
    """
    One or more document IDs returned by a [file upload](https://docs.stripe.com/api/persons/update#create_file) with a purpose value of `account_requirement`.
    """
    type: Literal["files"]
    """
    The format of the document. Currently supports `files` only.
    """


class PersonCreateParamsDocumentsPassport(TypedDict):
    files: List[str]
    """
    One or more document IDs returned by a [file upload](https://docs.stripe.com/api/persons/update#create_file) with a purpose value of `account_requirement`.
    """
    type: Literal["files"]
    """
    The format of the document. Currently supports `files` only.
    """


class PersonCreateParamsDocumentsPrimaryVerification(TypedDict):
    front_back: "PersonCreateParamsDocumentsPrimaryVerificationFrontBack"
    """
    The [file upload](https://docs.stripe.com/api/persons/update#create_file) tokens referring to each side of the document.
    """
    type: Literal["front_back"]
    """
    The format of the verification document. Currently supports `front_back` only.
    """


class PersonCreateParamsDocumentsPrimaryVerificationFrontBack(TypedDict):
    back: NotRequired[str]
    """
    A [file upload](https://docs.stripe.com/api/persons/update#create_file) token representing the back of the verification document. The purpose of the uploaded file should be 'identity_document'. The uploaded file needs to be a color image (smaller than 8,000px by 8,000px), in JPG, PNG, or PDF format, and less than 10 MB in size.
    """
    front: str
    """
    A [file upload](https://docs.stripe.com/api/persons/update#create_file) token representing the front of the verification document. The purpose of the uploaded file should be 'identity_document'. The uploaded file needs to be a color image (smaller than 8,000px by 8,000px), in JPG, PNG, or PDF format, and less than 10 MB in size.
    """


class PersonCreateParamsDocumentsSecondaryVerification(TypedDict):
    front_back: "PersonCreateParamsDocumentsSecondaryVerificationFrontBack"
    """
    The [file upload](https://docs.stripe.com/api/persons/update#create_file) tokens referring to each side of the document.
    """
    type: Literal["front_back"]
    """
    The format of the verification document. Currently supports `front_back` only.
    """


class PersonCreateParamsDocumentsSecondaryVerificationFrontBack(TypedDict):
    back: NotRequired[str]
    """
    A [file upload](https://docs.stripe.com/api/persons/update#create_file) token representing the back of the verification document. The purpose of the uploaded file should be 'identity_document'. The uploaded file needs to be a color image (smaller than 8,000px by 8,000px), in JPG, PNG, or PDF format, and less than 10 MB in size.
    """
    front: str
    """
    A [file upload](https://docs.stripe.com/api/persons/update#create_file) token representing the front of the verification document. The purpose of the uploaded file should be 'identity_document'. The uploaded file needs to be a color image (smaller than 8,000px by 8,000px), in JPG, PNG, or PDF format, and less than 10 MB in size.
    """


class PersonCreateParamsDocumentsVisa(TypedDict):
    files: List[str]
    """
    One or more document IDs returned by a [file upload](https://docs.stripe.com/api/persons/update#create_file) with a purpose value of `account_requirement`.
    """
    type: Literal["files"]
    """
    The format of the document. Currently supports `files` only.
    """


class PersonCreateParamsIdNumber(TypedDict):
    type: Literal[
        "ae_eid",
        "ao_nif",
        "ar_cuil",
        "ar_dni",
        "at_stn",
        "az_tin",
        "bd_brc",
        "bd_etin",
        "bd_nid",
        "be_nrn",
        "bg_ucn",
        "bm_pp",
        "bn_nric",
        "bo_ci",
        "br_cpf",
        "bt_cid",
        "ca_sin",
        "ch_oasi",
        "cl_rut",
        "cn_pp",
        "co_nuip",
        "cr_ci",
        "cr_cpf",
        "cr_dimex",
        "cr_nite",
        "cy_tic",
        "cz_rc",
        "de_stn",
        "dk_cpr",
        "do_cie",
        "do_rcn",
        "ec_ci",
        "ee_ik",
        "eg_tin",
        "es_nif",
        "fi_hetu",
        "fr_nir",
        "gb_nino",
        "gh_pin",
        "gr_afm",
        "gt_nit",
        "gy_tin",
        "hk_id",
        "hn_rtn",
        "hr_oib",
        "hu_ad",
        "id_nik",
        "ie_ppsn",
        "is_kt",
        "it_cf",
        "jm_trn",
        "jo_pin",
        "jp_inc",
        "ke_pin",
        "ky_pp",
        "kz_iin",
        "li_peid",
        "lk_nic",
        "lt_ak",
        "lu_nif",
        "lv_pk",
        "mo_bir",
        "mt_nic",
        "mv_tin",
        "mx_rfc",
        "my_nric",
        "mz_nuit",
        "ng_nin",
        "nl_bsn",
        "no_nin",
        "nz_ird",
        "pa_ruc",
        "pe_dni",
        "ph_tin",
        "pk_cnic",
        "pk_snic",
        "pl_pesel",
        "pt_nif",
        "py_ruc",
        "ro_cnp",
        "sa_tin",
        "se_pin",
        "sg_fin",
        "sg_nric",
        "si_pin",
        "sk_dic",
        "sv_nit",
        "th_lc",
        "th_pin",
        "tr_tin",
        "us_itin",
        "us_itin_last_4",
        "us_ssn",
        "us_ssn_last_4",
        "uy_dni",
        "vg_pp",
        "za_id",
    ]
    """
    The ID number type of an individual.
    """
    value: str
    """
    The value of the ID number.
    """


class PersonCreateParamsRelationship(TypedDict):
    authorizer: NotRequired[bool]
    """
    Whether the individual is an authorizer of the Account's identity.
    """
    director: NotRequired[bool]
    """
    Indicates whether the person is a director of the associated legal entity.
    """
    executive: NotRequired[bool]
    """
    Indicates whether the person is an executive of the associated legal entity.
    """
    legal_guardian: NotRequired[bool]
    """
    Indicates whether the person is a legal guardian of the associated legal entity.
    """
    owner: NotRequired[bool]
    """
    Indicates whether the person is an owner of the associated legal entity.
    """
    percent_ownership: NotRequired[Decimal]
    """
    The percentage of ownership the person has in the associated legal entity.
    """
    representative: NotRequired[bool]
    """
    Indicates whether the person is a representative of the associated legal entity.
    """
    title: NotRequired[str]
    """
    The title or position the person holds in the associated legal entity.
    """


class PersonCreateParamsScriptAddresses(TypedDict):
    kana: NotRequired["PersonCreateParamsScriptAddressesKana"]
    """
    Kana Address.
    """
    kanji: NotRequired["PersonCreateParamsScriptAddressesKanji"]
    """
    Kanji Address.
    """


class PersonCreateParamsScriptAddressesKana(TypedDict):
    city: NotRequired[str]
    """
    City, district, suburb, town, or village.
    """
    country: str
    """
    Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
    """
    line1: NotRequired[str]
    """
    Address line 1 (e.g., street, PO Box, or company name).
    """
    line2: NotRequired[str]
    """
    Address line 2 (e.g., apartment, suite, unit, or building).
    """
    postal_code: NotRequired[str]
    """
    ZIP or postal code.
    """
    state: NotRequired[str]
    """
    State, county, province, or region.
    """
    town: NotRequired[str]
    """
    Town or district.
    """


class PersonCreateParamsScriptAddressesKanji(TypedDict):
    city: NotRequired[str]
    """
    City, district, suburb, town, or village.
    """
    country: str
    """
    Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
    """
    line1: NotRequired[str]
    """
    Address line 1 (e.g., street, PO Box, or company name).
    """
    line2: NotRequired[str]
    """
    Address line 2 (e.g., apartment, suite, unit, or building).
    """
    postal_code: NotRequired[str]
    """
    ZIP or postal code.
    """
    state: NotRequired[str]
    """
    State, county, province, or region.
    """
    town: NotRequired[str]
    """
    Town or district.
    """


class PersonCreateParamsScriptNames(TypedDict):
    kana: NotRequired["PersonCreateParamsScriptNamesKana"]
    """
    Persons name in kana script.
    """
    kanji: NotRequired["PersonCreateParamsScriptNamesKanji"]
    """
    Persons name in kanji script.
    """


class PersonCreateParamsScriptNamesKana(TypedDict):
    given_name: NotRequired[str]
    """
    The person's first or given name.
    """
    surname: NotRequired[str]
    """
    The person's last or family name.
    """


class PersonCreateParamsScriptNamesKanji(TypedDict):
    given_name: NotRequired[str]
    """
    The person's first or given name.
    """
    surname: NotRequired[str]
    """
    The person's last or family name.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import List
from typing_extensions import Literal, NotRequired, TypedDict


class CustomerCreateFundingInstructionsParams(RequestOptions):
    bank_transfer: "CustomerCreateFundingInstructionsParamsBankTransfer"
    """
    Additional parameters for `bank_transfer` funding types
    """
    currency: str
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    funding_type: Literal["bank_transfer"]
    """
    The `funding_type` to get the instructions for.
    """


class CustomerCreateFundingInstructionsParamsBankTransfer(TypedDict):
    eu_bank_transfer: NotRequired[
        "CustomerCreateFundingInstructionsParamsBankTransferEuBankTransfer"
    ]
    """
    Configuration for eu_bank_transfer funding type.
    """
    requested_address_types: NotRequired[
        List[Literal["iban", "sort_code", "spei", "zengin"]]
    ]
    """
    List of address types that should be returned in the financial_addresses response. If not specified, all valid types will be returned.

    Permitted values include: `sort_code`, `zengin`, `iban`, or `spei`.
    """
    type: Literal[
        "eu_bank_transfer",
        "gb_bank_transfer",
        "jp_bank_transfer",
        "mx_bank_transfer",
        "us_bank_transfer",
    ]
    """
    The type of the `bank_transfer`
    """


class CustomerCreateFundingInstructionsParamsBankTransferEuBankTransfer(
    TypedDict,
):
    country: str
    """
    The desired country code of the bank account information. Permitted values include: `DE`, `FR`, `IE`, or `NL`.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing_extensions import Literal, NotRequired, TypedDict


class OutboundSetupIntentCreateParams(TypedDict):
    payout_method: NotRequired[str]
    """
    If provided, the existing payout method resource to link to this setup intent.
    Any payout_method_data provided is used to update information on this linked payout method resource.
    """
    payout_method_data: NotRequired[
        "OutboundSetupIntentCreateParamsPayoutMethodData"
    ]
    """
    If no payout_method provided, used to create the underlying credential that is set up for outbound money movement.
    If a payout_method provided, used to update data on the credential linked to this setup intent.
    """
    usage_intent: NotRequired[Literal["payment", "transfer"]]
    """
    Specify which type of outbound money movement this credential should be set up for (payment | transfer).
    If not provided, defaults to payment.
    """


class OutboundSetupIntentCreateParamsPayoutMethodData(TypedDict):
    bank_account: NotRequired[
        "OutboundSetupIntentCreateParamsPayoutMethodDataBankAccount"
    ]
    """
    The type specific details of the bank account payout method.
    """
    card: NotRequired["OutboundSetupIntentCreateParamsPayoutMethodDataCard"]
    """
    The type specific details of the card payout method.
    """
    crypto_wallet: NotRequired[
        "OutboundSetupIntentCreateParamsPayoutMethodDataCryptoWallet"
    ]
    """
    The type specific details of the crypto wallet payout method.
    """
    type: Literal["bank_account", "card", "crypto_wallet"]
    """
    Closed Enum. The type of payout method to be created.
    """


class OutboundSetupIntentCreateParamsPayoutMethodDataBankAccount(TypedDict):
    account_number: str
    """
    The account number or IBAN of the bank account.
    """
    bank_account_type: NotRequired[
        Literal["checking", "futsu", "savings", "toza"]
    ]
    """
    Closed Enum. The type of the bank account (checking or savings).
    """
    branch_number: NotRequired[str]
    """
    The branch number of the bank account, if present.
    """
    country: str
    """
    The country code of the bank account.
    """
    currency: str
    """
    The currency of the bank account.
    """
    routing_number: NotRequired[str]
    """
    The routing number of the bank account, if present.
    """
    swift_code: NotRequired[str]
    """
    The swift code of the bank account, if present.
    """


class OutboundSetupIntentCreateParamsPayoutMethodDataCard(TypedDict):
    currency: str
    """
    The currency of the card.
    """
    exp_month: str
    """
    The expiration month of the card.
    """
    exp_year: str
    """
    The expiration year of the card.
    """
    number: str
    """
    The card number.
    """


class OutboundSetupIntentCreateParamsPayoutMethodDataCryptoWallet(TypedDict):
    address: str
    """
    Crypto wallet address.
    """
    memo: NotRequired[str]
    """
    Optional field, required if network supports memos (only "stellar" currently).
    """
    network: Literal[
        "arbitrum",
        "avalanche_c_chain",
        "base",
        "ethereum",
        "optimism",
        "polygon",
        "solana",
        "stellar",
        "tempo",
    ]
    """
    Which rail we should use to make an Outbound money movement to this wallet.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing_extensions import Literal, NotRequired, TypedDict


class RateCardSubscriptionListParams(TypedDict):
    billing_cadence: NotRequired[str]
    """
    Optionally filter by a BillingCadence. Mutually exclusive with `payers`, `rate_card`, and `rate_card_version`.
    """
    limit: NotRequired[int]
    """
    The page size limit. Defaults to 20.
    """
    payer: NotRequired["RateCardSubscriptionListParamsPayer"]
    """
    Optionally filter by the payer associated with Billing Cadences which the Rate Card Subscriptions are subscribed to.
    Mutually exclusive with `billing_cadence`, `rate_card`, and `rate_card_version`.
    """
    rate_card: NotRequired[str]
    """
    Optionally filter by a RateCard. Mutually exclusive with `billing_cadence`, `payers`, and `rate_card_version`.
    """
    rate_card_version: NotRequired[str]
    """
    Optionally filter by a RateCard version. Mutually exclusive with `billing_cadence`, `payers`, and `rate_card`.
    """
    servicing_status: NotRequired[
        Literal["active", "canceled", "paused", "pending"]
    ]
    """
    Optionally filter by servicing status.
    """


class RateCardSubscriptionListParamsPayer(TypedDict):
    customer: NotRequired[str]
    """
    The ID of the Customer object. If provided, only the Rate Card Subscriptions that are subscribed on the Billing Cadences with the specified payer will be returned.
    """
    type: Literal["customer"]
    """
    A string identifying the type of the payer. Currently the only supported value is `customer`.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing_extensions import NotRequired, TypedDict


class UsBankAccountUpdateParams(TypedDict):
    fedwire_routing_number: NotRequired[str]
    """
    The bank account's Fedwire routing number can be provided for update if it was empty previously.
    """
    routing_number: NotRequired[str]
    """
    The bank account's ACH routing number can be provided for update if it was empty previously.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from stripe._stripe_object import UntypedStripeObject
from typing import Dict, List
from typing_extensions import Literal, NotRequired, TypedDict


class CreditUnderwritingRecordCreateFromProactiveReviewParams(RequestOptions):
    credit_user: (
        "CreditUnderwritingRecordCreateFromProactiveReviewParamsCreditUser"
    )
    """
    Information about the company or person applying or holding the account.
    """
    decided_at: int
    """
    Date when a decision was made.
    """
    decision: "CreditUnderwritingRecordCreateFromProactiveReviewParamsDecision"
    """
    Details about the decision.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    metadata: NotRequired["Dict[str, str]|UntypedStripeObject[str]"]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    regulatory_reporting_file: NotRequired[str]
    """
    File containing regulatory reporting data for the decision. Required if you are subject to this [reporting requirement](https://docs.stripe.com/issuing/credit/report-required-regulatory-data-for-credit-decisions).
    """
    underwriting_exception: NotRequired[
        "CreditUnderwritingRecordCreateFromProactiveReviewParamsUnderwritingException"
    ]
    """
    If an exception to the usual underwriting criteria was made for this decision, details about the exception must be provided. Exceptions should only be granted in rare circumstances, in consultation with Stripe Compliance.
    """


class CreditUnderwritingRecordCreateFromProactiveReviewParamsCreditUser(
    TypedDict,
):
    email: str
    """
    Email of the applicant or accountholder.
    """
    name: str
    """
    Full name of the company or person.
    """


class CreditUnderwritingRecordCreateFromProactiveReviewParamsDecision(
    TypedDict,
):
    credit_limit_approved: NotRequired[
        "CreditUnderwritingRecordCreateFromProactiveReviewParamsDecisionCreditLimitApproved"
    ]
    """
    Details about the credit limit approved. An approved credit limit is required before you can set a `credit_limit_amount` in the [CreditPolicy API](https://docs.stripe.com/api/issuing/credit_policy/)
    """
    credit_limit_decreased: NotRequired[
        "CreditUnderwritingRecordCreateFromProactiveReviewParamsDecisionCreditLimitDecreased"
    ]
    """
    Details about the credit limit decreased.
    """
    credit_line_closed: NotRequired[
        "CreditUnderwritingRecordCreateFromProactiveReviewParamsDecisionCreditLineClosed"
    ]
    """
    Details about the credit line closed.
    """
    type: Literal[
        "credit_limit_approved",
        "credit_limit_decreased",
        "credit_line_closed",
        "no_changes",
    ]
    """
    Outcome of the decision.
    """


class CreditUnderwritingRecordCreateFromProactiveReviewParamsDecisionCreditLimitApproved(
    TypedDict,
):
    amount: int
    """
    The credit approved, in the currency of the account and [smallest currency unit](https://docs.stripe.com/currencies#zero-decimal).
    """
    currency: NotRequired[str]
    """
    The currency of the credit approved, will default to the Account's Issuing currency.
    """


class CreditUnderwritingRecordCreateFromProactiveReviewParamsDecisionCreditLimitDecreased(
    TypedDict,
):
    amount: int
    """
    The credit approved, in the currency of the account and [smallest currency unit](https://docs.stripe.com/currencies#zero-decimal).
    """
    currency: NotRequired[str]
    """
    The currency of the credit approved, will default to the Account's Issuing currency.
    """
    reason_other_explanation: NotRequired[str]
    """
    Details about the `reasons.other` when present.
    """
    reasons: List[
        Literal[
            "applicant_is_not_beneficial_owner",
            "applicant_too_young",
            "application_is_not_beneficial_owner",
            "bankruptcy",
            "business_size_too_small",
            "change_in_financial_state",
            "change_in_utilization_of_credit_line",
            "current_account_tier_ineligible",
            "customer_already_exists",
            "customer_requested_account_closure",
            "debt_to_cash_balance_ratio_too_high",
            "debt_to_equity_ratio_too_high",
            "decrease_in_income_to_expense_ratio",
            "decrease_in_social_media_performance",
            "delinquent_credit_obligations",
            "dispute_rate_too_high",
            "duration_of_residence",
            "exceeds_acceptable_platform_exposure",
            "excessive_income_or_revenue_obligations",
            "expenses_to_cash_balance_ratio_too_high",
            "foreclosure_or_repossession",
            "frozen_file_at_credit_bureau",
            "garnishment_or_attachment",
            "government_loan_program_criteria",
            "has_recent_credit_limit_increase",
            "high_concentration_of_clients",
            "high_risk_industry",
            "incomplete_application",
            "inconsistent_monthly_revenues",
            "insufficient_account_history_with_platform",
            "insufficient_bank_account_history",
            "insufficient_cash_balance",
            "insufficient_cash_flow",
            "insufficient_collateral",
            "insufficient_credit_experience",
            "insufficient_credit_utilization",
            "insufficient_deposits",
            "insufficient_income",
            "insufficient_margin_ratio",
            "insufficient_operating_profit",
            "insufficient_period_in_operation",
            "insufficient_reserves",
            "insufficient_revenue",
            "insufficient_social_media_performance",
            "insufficient_time_in_network",
            "insufficient_trade_credit_insurance",
            "insufficient_usage_as_qualified_expenses",
            "invalid_business_license",
            "lacking_cash_account",
            "late_payment_history_reported_to_bureau",
            "lien_collection_action_or_judgement",
            "negative_public_information",
            "no_credit_file",
            "other",
            "outside_supported_country",
            "outside_supported_state",
            "poor_payment_history_with_platform",
            "prior_or_current_legal_action",
            "prohibited_industry",
            "rate_of_cash_balance_fluctuation_too_high",
            "recent_inquiries_on_business_credit_report",
            "removal_of_bank_account_connection",
            "revenue_discrepancy",
            "runway_too_short",
            "suspected_fraud",
            "too_many_non_sufficient_funds_or_overdrafts",
            "unable_to_verify_address",
            "unable_to_verify_identity",
            "unable_to_verify_income_or_revenue",
            "unprofitable",
            "unsupportable_business_type",
        ]
    ]
    """
    List of reasons why the existing credit was decreased, up to 4 reasons, in order of importance.
    """


class CreditUnderwritingRecordCreateFromProactiveReviewParamsDecisionCreditLineClosed(
    TypedDict,
):
    reason_other_explanation: NotRequired[str]
    """
    Details about the `reasons.other` when present.
    """
    reasons: List[
        Literal[
            "applicant_is_not_beneficial_owner",
            "applicant_too_young",
            "application_is_not_beneficial_owner",
            "bankruptcy",
            "business_size_too_small",
            "change_in_financial_state",
            "change_in_utilization_of_credit_line",
            "current_account_tier_ineligible",
            "customer_already_exists",
            "customer_requested_account_closure",
            "debt_to_cash_balance_ratio_too_high",
            "debt_to_equity_ratio_too_high",
            "decrease_in_income_to_expense_ratio",
            "decrease_in_social_media_performance",
            "delinquent_credit_obligations",
            "dispute_rate_too_high",
            "duration_of_residence",
            "exceeds_acceptable_platform_exposure",
            "excessive_income_or_revenue_obligations",
            "expenses_to_cash_balance_ratio_too_high",
            "foreclosure_or_repossession",
            "frozen_file_at_credit_bureau",
            "garnishment_or_attachment",
            "government_loan_program_criteria",
            "has_recent_credit_limit_increase",
            "high_concentration_of_clients",
            "high_risk_industry",
            "incomplete_application",
            "inconsistent_monthly_revenues",
            "insufficient_account_history_with_platform",
            "insufficient_bank_account_history",
            "insufficient_cash_balance",
            "insufficient_cash_flow",
            "insufficient_collateral",
            "insufficient_credit_experience",
            "insufficient_credit_utilization",
            "insufficient_deposits",
            "insufficient_income",
            "insufficient_margin_ratio",
            "insufficient_operating_profit",
            "insufficient_period_in_operation",
            "insufficient_reserves",
            "insufficient_revenue",
            "insufficient_social_media_performance",
            "insufficient_time_in_network",
            "insufficient_trade_credit_insurance",
            "insufficient_usage_as_qualified_expenses",
            "invalid_business_license",
            "lacking_cash_account",
            "late_payment_history_reported_to_bureau",
            "lien_collection_action_or_judgement",
            "negative_public_information",
            "no_credit_file",
            "other",
            "outside_supported_country",
            "outside_supported_state",
            "poor_payment_history_with_platform",
            "prior_or_current_legal_action",
            "prohibited_industry",
            "rate_of_cash_balance_fluctuation_too_high",
            "recent_inquiries_on_business_credit_report",
            "removal_of_bank_account_connection",
            "revenue_discrepancy",
            "runway_too_short",
            "suspected_fraud",
            "too_many_non_sufficient_funds_or_overdrafts",
            "unable_to_verify_address",
            "unable_to_verify_identity",
            "unable_to_verify_income_or_revenue",
            "unprofitable",
            "unsupportable_business_type",
        ]
    ]
    """
    List of reasons why the credit line was closed, up to 4 reasons, in order of importance.
    """


class CreditUnderwritingRecordCreateFromProactiveReviewParamsUnderwritingException(
    TypedDict,
):
    explanation: str
    """
    Written explanation for the exception.
    """
    original_decision_type: Literal[
        "additional_information_requested",
        "application_rejected",
        "credit_limit_approved",
        "credit_limit_decreased",
        "credit_line_closed",
        "no_changes",
        "withdrawn_by_applicant",
    ]
    """
    The decision before the exception was applied.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from decimal import Decimal
from stripe.v2._amount import AmountParam
from typing import List
from typing_extensions import Literal, NotRequired, TypedDict


class ServiceActionCreateParams(TypedDict):
    credit_grant: NotRequired["ServiceActionCreateParamsCreditGrant"]
    """
    Details for the credit grant. Required if `type` is `credit_grant`.
    """
    credit_grant_per_tenant: NotRequired[
        "ServiceActionCreateParamsCreditGrantPerTenant"
    ]
    """
    Details for the credit grant per tenant. Required if `type` is `credit_grant_per_tenant`.
    """
    lookup_key: NotRequired[str]
    """
    An internal key you can use to search for this service action. Maximum length of 200 characters.
    """
    service_interval: Literal["day", "month", "week", "year"]
    """
    The interval for assessing service.
    """
    service_interval_count: int
    """
    The length of the interval for assessing service.
    """
    type: Literal["credit_grant", "credit_grant_per_tenant"]
    """
    The type of the service action.
    """


class ServiceActionCreateParamsCreditGrant(TypedDict):
    amount: "ServiceActionCreateParamsCreditGrantAmount"
    """
    The amount of the credit grant.
    """
    applicability_config: (
        "ServiceActionCreateParamsCreditGrantApplicabilityConfig"
    )
    """
    Defines the scope where the credit grant is applicable.
    """
    category: NotRequired[Literal["paid", "promotional"]]
    """
    The category of the credit grant.
    """
    expiry_config: "ServiceActionCreateParamsCreditGrantExpiryConfig"
    """
    The expiry configuration for the credit grant.
    """
    name: str
    """
    A descriptive name shown in dashboard.
    """
    priority: NotRequired[int]
    """
    The desired priority for applying this credit grant. If not specified, it will be set to the default value of 50. The highest priority is 0 and the lowest is 100.
    """


class ServiceActionCreateParamsCreditGrantAmount(TypedDict):
    custom_pricing_unit: NotRequired[
        "ServiceActionCreateParamsCreditGrantAmountCustomPricingUnit"
    ]
    """
    The custom pricing unit amount of the credit grant. Required if `type` is `custom_pricing_unit`.
    """
    monetary: NotRequired[AmountParam]
    """
    The monetary amount of the credit grant. Required if `type` is `monetary`.
    """
    type: Literal["custom_pricing_unit", "monetary"]
    """
    The type of the credit grant amount. We currently support `monetary` and `custom_pricing_unit` billing credits.
    """


class ServiceActionCreateParamsCreditGrantAmountCustomPricingUnit(TypedDict):
    id: str
    """
    The id of the custom pricing unit.
    """
    value: Decimal
    """
    The value of the credit grant, decimal value represented as a string.
    """


class ServiceActionCreateParamsCreditGrantApplicabilityConfig(TypedDict):
    scope: "ServiceActionCreateParamsCreditGrantApplicabilityConfigScope"
    """
    The applicability scope of the credit grant.
    """


class ServiceActionCreateParamsCreditGrantApplicabilityConfigScope(TypedDict):
    billable_items: NotRequired[List[str]]
    """
    The billable items to apply the credit grant to.
    """
    price_type: NotRequired[Literal["metered"]]
    """
    The price type that credit grants can apply to. Stripe supports the `metered` price type, which applies to metered prices and rate cards. Cannot be used in combination with `billable_items`.
    """


class ServiceActionCreateParamsCreditGrantExpiryConfig(TypedDict):
    type: Literal["end_of_service_period"]
    """
    The type of the expiry configuration. We currently support `end_of_service_period`.
    """


class ServiceActionCreateParamsCreditGrantPerTenant(TypedDict):
    amount: "ServiceActionCreateParamsCreditGrantPerTenantAmount"
    """
    The amount of the credit grant.
    """
    applicability_config: (
        "ServiceActionCreateParamsCreditGrantPerTenantApplicabilityConfig"
    )
    """
    Defines the scope where the credit grant is applicable.
    """
    category: NotRequired[Literal["paid", "promotional"]]
    """
    The category of the credit grant.
    """
    expiry_config: "ServiceActionCreateParamsCreditGrantPerTenantExpiryConfig"
    """
    The expiry configuration for the credit grant.
    """
    grant_condition: (
        "ServiceActionCreateParamsCreditGrantPerTenantGrantCondition"
    )
    """
    The grant condition for the credit grant.
    """
    name: str
    """
    Customer-facing name for the credit grant.
    """
    priority: NotRequired[int]
    """
    The desired priority for applying this credit grant. If not specified, it will be set to the default value of 50. The highest priority is 0 and the lowest is 100.
    """


class ServiceActionCreateParamsCreditGrantPerTenantAmount(TypedDict):
    custom_pricing_unit: NotRequired[
        "ServiceActionCreateParamsCreditGrantPerTenantAmountCustomPricingUnit"
    ]
    """
    The custom pricing unit amount of the credit grant. Required if `type` is `custom_pricing_unit`.
    """
    monetary: NotRequired[AmountParam]
    """
    The monetary amount of the credit grant. Required if `type` is `monetary`.
    """
    type: Literal["custom_pricing_unit", "monetary"]
    """
    The type of the credit grant amount. We currently support `monetary` and `custom_pricing_unit` billing credits.
    """


class ServiceActionCreateParamsCreditGrantPerTenantAmountCustomPricingUnit(
    TypedDict,
):
    id: str
    """
    The id of the custom pricing unit.
    """
    value: Decimal
    """
    The value of the credit grant, decimal value represented as a string.
    """


class ServiceActionCreateParamsCreditGrantPerTenantApplicabilityConfig(
    TypedDict,
):
    scope: (
        "ServiceActionCreateParamsCreditGrantPerTenantApplicabilityConfigScope"
    )
    """
    The applicability scope of the credit grant.
    """


class ServiceActionCreateParamsCreditGrantPerTenantApplicabilityConfigScope(
    TypedDict,
):
    billable_items: NotRequired[List[str]]
    """
    The billable items to apply the credit grant to.
    """
    price_type: NotRequired[Literal["metered"]]
    """
    The price type that credit grants can apply to. Stripe supports the `metered` price type, which applies to metered prices and rate cards. Cannot be used in combination with `billable_items`.
    """


class ServiceActionCreateParamsCreditGrantPerTenantExpiryConfig(TypedDict):
    type: Literal["end_of_service_period"]
    """
    The type of the expiry configuration. We currently support `end_of_service_period`.
    """


class ServiceActionCreateParamsCreditGrantPerTenantGrantCondition(TypedDict):
    meter_event_first_per_period: NotRequired[
        "ServiceActionCreateParamsCreditGrantPerTenantGrantConditionMeterEventFirstPerPeriod"
    ]
    """
    The grant condition for the meter event first per period.
    """
    type: Literal["meter_event_first_per_period"]
    """
    The type of the grant condition. We currently support `meter_event_first_per_period`.
    """


class ServiceActionCreateParamsCreditGrantPerTenantGrantConditionMeterEventFirstPerPeriod(
    TypedDict,
):
    meter_segment_conditions: List[
        "ServiceActionCreateParamsCreditGrantPerTenantGrantConditionMeterEventFirstPerPeriodMeterSegmentCondition"
    ]
    """
    The meter segment conditions for the grant condition.
    """


class ServiceActionCreateParamsCreditGrantPerTenantGrantConditionMeterEventFirstPerPeriodMeterSegmentCondition(
    TypedDict,
):
    dimension: NotRequired[
        "ServiceActionCreateParamsCreditGrantPerTenantGrantConditionMeterEventFirstPerPeriodMeterSegmentConditionDimension"
    ]
    """
    Dimension-based meter segment condition.
    """
    type: Literal["dimension"]
    """
    The type of the meter segment condition. We currently support `dimension`.
    """


class ServiceActionCreateParamsCreditGrantPerTenantGrantConditionMeterEventFirstPerPeriodMeterSegmentConditionDimension(
    TypedDict,
):
    payload_key: str
    """
    The payload key for the dimension.
    """
    value: str
    """
    The value for the dimension.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from decimal import Decimal
from stripe._stripe_object import UntypedStripeObject
from typing import Dict, List
from typing_extensions import Literal, NotRequired, TypedDict


class RateCreateParams(TypedDict):
    custom_pricing_unit_amount: NotRequired[
        "RateCreateParamsCustomPricingUnitAmount"
    ]
    """
    The custom pricing unit that this rate binds to. One of `unit_amount`, `tiers`, or `custom_pricing_unit_amount` is required.
    """
    metadata: NotRequired["Dict[str, str]|UntypedStripeObject[str]"]
    """
    Set of [key-value pairs](https://docs.stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    metered_item: str
    """
    The Metered Item that this rate binds to.
    """
    tiering_mode: NotRequired[Literal["graduated", "volume"]]
    """
    Defines whether the tiered price is graduated or volume-based. In volume-based tiering, the maximum
    quantity within a period determines the per-unit price. In graduated tiering, the pricing changes as the quantity
    grows into new tiers. Can only be set if `tiers` is set.
    """
    tiers: NotRequired[List["RateCreateParamsTier"]]
    """
    Each element represents a pricing tier. One of `unit_amount`, `tiers`, or `custom_pricing_unit_amount` is required.
    """
    transform_quantity: NotRequired["RateCreateParamsTransformQuantity"]
    """
    Apply a transformation to the reported usage or set quantity before computing the amount billed.
    """
    unit_amount: NotRequired[str]
    """
    The per-unit amount to be charged, represented as a decimal string in minor currency units with at most 12 decimal
    places. One of `unit_amount`, `tiers`, or `custom_pricing_unit_amount` is required.
    """


class RateCreateParamsCustomPricingUnitAmount(TypedDict):
    id: str
    """
    The id of the custom pricing unit.
    """
    value: str
    """
    The unit value for the custom pricing unit, as a string.
    """


class RateCreateParamsTier(TypedDict):
    flat_amount: NotRequired[str]
    """
    Price for the entire tier, represented as a decimal string in minor currency units with at most 12 decimal places.
    """
    unit_amount: NotRequired[str]
    """
    Per-unit price for units included in this tier, represented as a decimal string in minor currency units with at
    most 12 decimal places.
    """
    up_to_decimal: NotRequired[Decimal]
    """
    Up to and including this quantity is contained in the tier. Only one of `up_to_decimal` and `up_to_inf` may
    be set.
    """
    up_to_inf: NotRequired[Literal["inf"]]
    """
    No upper bound to this tier. Only one of `up_to_decimal` and `up_to_inf` may be set.
    """


class RateCreateParamsTransformQuantity(TypedDict):
    divide_by: int
    """
    Divide usage by this number.
    """
    round: Literal["down", "up"]
    """
    After division, round the result up or down.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import UntypedStripeObject
from typing import Dict, List, Optional
from typing_extensions import Literal, NotRequired, TypedDict


class CadenceUpdateParams(TypedDict):
    include: NotRequired[
        List[Literal["invoice_discount_rules", "settings_data"]]
    ]
    """
    Additional resource to include in the response.
    """
    lookup_key: NotRequired[str]
    """
    A lookup key used to retrieve cadences dynamically from a static string. Maximum length of 200 characters.
    """
    metadata: NotRequired[
        "Dict[str, Optional[str]]|UntypedStripeObject[Optional[str]]"
    ]
    """
    Set of [key-value pairs](https://docs.stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    payer: NotRequired["CadenceUpdateParamsPayer"]
    """
    The payer determines the entity financially responsible for the bill.
    """
    settings: NotRequired["CadenceUpdateParamsSettings"]
    """
    The settings associated with the cadence.
    """


class CadenceUpdateParamsPayer(TypedDict):
    billing_profile: NotRequired[str]
    """
    The ID of the Billing Profile object which determines how a bill is paid.
    """


class CadenceUpdateParamsSettings(TypedDict):
    bill: NotRequired["CadenceUpdateParamsSettingsBill"]
    """
    Settings that configure bills generation, which includes calculating totals, tax, and presenting invoices. If null is provided, the current bill settings will be removed from the billing cadence.
    """
    collection: NotRequired["CadenceUpdateParamsSettingsCollection"]
    """
    Settings that configure and manage the behavior of collecting payments. If null is provided, the current collection settings will be removed from the billing cadence.
    """


class CadenceUpdateParamsSettingsBill(TypedDict):
    id: str
    """
    The ID of the referenced settings object.
    """
    version: NotRequired[str]
    """
    An optional field to specify the version of Settings to use.
    If not provided, this defaults to the `live_version` specified on the setting, any time the settings are used.
    Using a specific version here prevents the settings from updating, and is discouraged for cadences.
    To clear a pinned version, set the version to null.
    """


class CadenceUpdateParamsSettingsCollection(TypedDict):
    id: str
    """
    The ID of the referenced settings object.
    """
    version: NotRequired[str]
    """
    An optional field to specify the version of Settings to use.
    If not provided, this defaults to the `live_version` specified on the setting, any time the settings are used.
    Using a specific version here prevents the settings from updating, and is discouraged for cadences.
    To clear a pinned version, set the version to null.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._expandable_field import ExpandableField
from stripe._list_object import ListObject
from stripe._listable_api_resource import ListableAPIResource
from stripe._stripe_object import StripeObject, UntypedStripeObject
from typing import ClassVar, Optional
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._dispute import Dispute
    from stripe._refund import Refund
    from stripe.params.reserve._release_list_params import ReleaseListParams
    from stripe.params.reserve._release_retrieve_params import (
        ReleaseRetrieveParams,
    )
    from stripe.reserve._hold import Hold
    from stripe.reserve._plan import Plan


class Release(ListableAPIResource["Release"]):
    """
    ReserveReleases represent the release of funds from a ReserveHold.
    """

    OBJECT_NAME: ClassVar[Literal["reserve.release"]] = "reserve.release"

    class SourceTransaction(StripeObject):
        dispute: Optional[ExpandableField["Dispute"]]
        """
        The ID of the dispute.
        """
        refund: Optional[ExpandableField["Refund"]]
        """
        The ID of the refund.
        """
        type: Literal["dispute", "refund"]
        """
        The type of source transaction.
        """

    amount: int
    """
    Amount released. A positive integer representing how much is released in the [smallest currency unit](https://docs.stripe.com/currencies#zero-decimal).
    """
    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    created_by: Literal["application", "stripe"]
    """
    Indicates which party created this ReserveRelease.
    """
    currency: str
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    metadata: Optional[UntypedStripeObject[str]]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    object: Literal["reserve.release"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    reason: Literal[
        "bulk_hold_expiry",
        "hold_released_early",
        "hold_reversed",
        "plan_disabled",
    ]
    """
    The reason for the ReserveRelease, indicating why the funds were released.
    """
    released_at: int
    """
    The release timestamp of the funds.
    """
    reserve_hold: Optional[ExpandableField["Hold"]]
    """
    The ReserveHold this ReserveRelease is associated with.
    """
    reserve_plan: Optional[ExpandableField["Plan"]]
    """
    The ReservePlan ID this ReserveRelease is associated with. This field is only populated if a ReserveRelease is created by a ReservePlan disable operation, or from a scheduled ReservedHold expiry.
    """
    source_transaction: Optional[SourceTransaction]

    @classmethod
    def list(
        cls, **params: Unpack["ReleaseListParams"]
    ) -> ListObject["Release"]:
        """
        Returns a list of ReserveReleases previously created. The ReserveReleases are returned in sorted order, with the most recent ReserveReleases appearing first.
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    async def list_async(
        cls, **params: Unpack["ReleaseListParams"]
    ) -> ListObject["Release"]:
        """
        Returns a list of ReserveReleases previously created. The ReserveReleases are returned in sorted order, with the most recent ReserveReleases appearing first.
        """
        result = await cls._static_request_async(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["ReleaseRetrieveParams"]
    ) -> "Release":
        """
        Retrieve a ReserveRelease.
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["ReleaseRetrieveParams"]
    ) -> "Release":
        """
        Retrieve a ReserveRelease.
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    _inner_class_types = {"source_transaction": SourceTransaction}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from typing import Optional
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.params.v2.billing._meter_event_stream_create_params import (
        MeterEventStreamCreateParams,
    )


class MeterEventStreamService(StripeService):
    def create(
        self,
        params: "MeterEventStreamCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "None":
        """
        Creates meter events. Events are processed asynchronously, including validation. Requires a meter event session for authentication. Supports up to 10,000 requests per second in livemode. For even higher rate-limits, contact sales.
        """
        self._request(
            "post",
            "/v2/billing/meter_event_stream",
            base_address="meter_events",
            params=params,
            options=options,
        )

    async def create_async(
        self,
        params: "MeterEventStreamCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "None":
        """
        Creates meter events. Events are processed asynchronously, including validation. Requires a meter event session for authentication. Supports up to 10,000 requests per second in livemode. For even higher rate-limits, contact sales.
        """
        await self._request_async(
            "post",
            "/v2/billing/meter_event_stream",
            base_address="meter_events",
            params=params,
            options=options,
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._createable_api_resource import CreateableAPIResource
from stripe._customer import Customer
from stripe._error import InvalidRequestError
from stripe._list_object import ListObject
from stripe._stripe_object import StripeObject, UntypedStripeObject
from stripe._updateable_api_resource import UpdateableAPIResource
from stripe._util import class_method_variant, sanitize_id
from typing import ClassVar, List, Optional, cast, overload
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._source_transaction import SourceTransaction
    from stripe.params._source_create_params import SourceCreateParams
    from stripe.params._source_list_source_transactions_params import (
        SourceListSourceTransactionsParams,
    )
    from stripe.params._source_modify_params import SourceModifyParams
    from stripe.params._source_retrieve_params import SourceRetrieveParams
    from stripe.params._source_verify_params import SourceVerifyParams


class Source(CreateableAPIResource["Source"], UpdateableAPIResource["Source"]):
    """
    `Source` objects allow you to accept a variety of payment methods. They
    represent a customer's payment instrument, and can be used with the Stripe API
    just like a `Card` object: once chargeable, they can be charged, or can be
    attached to customers.

    Stripe doesn't recommend using the deprecated [Sources API](https://docs.stripe.com/api/sources).
    We recommend that you adopt the [PaymentMethods API](https://docs.stripe.com/api/payment_methods).
    This newer API provides access to our latest features and payment method types.

    Related guides: [Sources API](https://docs.stripe.com/sources) and [Sources & Customers](https://docs.stripe.com/sources/customers).
    """

    OBJECT_NAME: ClassVar[Literal["source"]] = "source"

    class AchCreditTransfer(StripeObject):
        account_number: Optional[str]
        bank_name: Optional[str]
        fingerprint: Optional[str]
        refund_account_holder_name: Optional[str]
        refund_account_holder_type: Optional[str]
        refund_routing_number: Optional[str]
        routing_number: Optional[str]
        swift_code: Optional[str]

    class AchDebit(StripeObject):
        bank_name: Optional[str]
        country: Optional[str]
        fingerprint: Optional[str]
        last4: Optional[str]
        routing_number: Optional[str]
        type: Optional[str]

    class AcssDebit(StripeObject):
        bank_address_city: Optional[str]
        bank_address_line_1: Optional[str]
        bank_address_line_2: Optional[str]
        bank_address_postal_code: Optional[str]
        bank_name: Optional[str]
        category: Optional[str]
        country: Optional[str]
        fingerprint: Optional[str]
        last4: Optional[str]
        routing_number: Optional[str]

    class Alipay(StripeObject):
        data_string: Optional[str]
        native_url: Optional[str]
        statement_descriptor: Optional[str]

    class AuBecsDebit(StripeObject):
        bsb_number: Optional[str]
        fingerprint: Optional[str]
        last4: Optional[str]

    class Bancontact(StripeObject):
        bank_code: Optional[str]
        bank_name: Optional[str]
        bic: Optional[str]
        iban_last4: Optional[str]
        preferred_language: Optional[str]
        statement_descriptor: Optional[str]

    class Card(StripeObject):
        address_line1_check: Optional[str]
        address_zip_check: Optional[str]
        brand: Optional[str]
        brand_product: Optional[str]
        country: Optional[str]
        cvc_check: Optional[str]
        description: Optional[str]
        dynamic_last4: Optional[str]
        exp_month: Optional[int]
        exp_year: Optional[int]
        fingerprint: Optional[str]
        funding: Optional[str]
        iin: Optional[str]
        issuer: Optional[str]
        last4: Optional[str]
        name: Optional[str]
        three_d_secure: Optional[str]
        tokenization_method: Optional[str]

    class CardPresent(StripeObject):
        application_cryptogram: Optional[str]
        application_preferred_name: Optional[str]
        authorization_code: Optional[str]
        authorization_response_code: Optional[str]
        brand: Optional[str]
        brand_product: Optional[str]
        country: Optional[str]
        cvm_type: Optional[str]
        data_type: Optional[str]
        dedicated_file_name: Optional[str]
        description: Optional[str]
        emv_auth_data: Optional[str]
        evidence_customer_signature: Optional[str]
        evidence_transaction_certificate: Optional[str]
        exp_month: Optional[int]
        exp_year: Optional[int]
        fingerprint: Optional[str]
        funding: Optional[str]
        iin: Optional[str]
        issuer: Optional[str]
        last4: Optional[str]
        pos_device_id: Optional[str]
        pos_entry_mode: Optional[str]
        read_method: Optional[str]
        reader: Optional[str]
        terminal_verification_results: Optional[str]
        transaction_status_information: Optional[str]

    class CodeVerification(StripeObject):
        attempts_remaining: int
        """
        The number of attempts remaining to authenticate the source object with a verification code.
        """
        status: str
        """
        The status of the code verification, either `pending` (awaiting verification, `attempts_remaining` should be greater than 0), `succeeded` (successful verification) or `failed` (failed verification, cannot be verified anymore as `attempts_remaining` should be 0).
        """

    class Eps(StripeObject):
        reference: Optional[str]
        statement_descriptor: Optional[str]

    class Giropay(StripeObject):
        bank_code: Optional[str]
        bank_name: Optional[str]
        bic: Optional[str]
        statement_descriptor: Optional[str]

    class Ideal(StripeObject):
        bank: Optional[str]
        bic: Optional[str]
        iban_last4: Optional[str]
        statement_descriptor: Optional[str]

    class Klarna(StripeObject):
        background_image_url: Optional[str]
        client_token: Optional[str]
        first_name: Optional[str]
        last_name: Optional[str]
        locale: Optional[str]
        logo_url: Optional[str]
        page_title: Optional[str]
        pay_later_asset_urls_descriptive: Optional[str]
        pay_later_asset_urls_standard: Optional[str]
        pay_later_name: Optional[str]
        pay_later_redirect_url: Optional[str]
        pay_now_asset_urls_descriptive: Optional[str]
        pay_now_asset_urls_standard: Optional[str]
        pay_now_name: Optional[str]
        pay_now_redirect_url: Optional[str]
        pay_over_time_asset_urls_descriptive: Optional[str]
        pay_over_time_asset_urls_standard: Optional[str]
        pay_over_time_name: Optional[str]
        pay_over_time_redirect_url: Optional[str]
        payment_method_categories: Optional[str]
        purchase_country: Optional[str]
        purchase_type: Optional[str]
        redirect_url: Optional[str]
        shipping_delay: Optional[int]
        shipping_first_name: Optional[str]
        shipping_last_name: Optional[str]

    class Multibanco(StripeObject):
        entity: Optional[str]
        reference: Optional[str]
        refund_account_holder_address_city: Optional[str]
        refund_account_holder_address_country: Optional[str]
        refund_account_holder_address_line1: Optional[str]
        refund_account_holder_address_line2: Optional[str]
        refund_account_holder_address_postal_code: Optional[str]
        refund_account_holder_address_state: Optional[str]
        refund_account_holder_name: Optional[str]
        refund_iban: Optional[str]

    class Owner(StripeObject):
        class Address(StripeObject):
            city: Optional[str]
            """
            City, district, suburb, town, or village.
            """
            country: Optional[str]
            """
            Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
            """
            line1: Optional[str]
            """
            Address line 1, such as the street, PO Box, or company name.
            """
            line2: Optional[str]
            """
            Address line 2, such as the apartment, suite, unit, or building.
            """
            postal_code: Optional[str]
            """
            ZIP or postal code.
            """
            state: Optional[str]
            """
            State, county, province, or region ([ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2)).
            """

        class VerifiedAddress(StripeObject):
            city: Optional[str]
            """
            City, district, suburb, town, or village.
            """
            country: Optional[str]
            """
            Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
            """
            line1: Optional[str]
            """
            Address line 1, such as the street, PO Box, or company name.
            """
            line2: Optional[str]
            """
            Address line 2, such as the apartment, suite, unit, or building.
            """
            postal_code: Optional[str]
            """
            ZIP or postal code.
            """
            state: Optional[str]
            """
            State, county, province, or region ([ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2)).
            """

        address: Optional[Address]
        """
        Owner's address.
        """
        email: Optional[str]
        """
        Owner's email address.
        """
        name: Optional[str]
        """
        Owner's full name.
        """
        phone: Optional[str]
        """
        Owner's phone number (including extension).
        """
        verified_address: Optional[VerifiedAddress]
        """
        Verified owner's address. Verified values are verified or provided by the payment method directly (and if supported) at the time of authorization or settlement. They cannot be set or mutated.
        """
        verified_email: Optional[str]
        """
        Verified owner's email address. Verified values are verified or provided by the payment method directly (and if supported) at the time of authorization or settlement. They cannot be set or mutated.
        """
        verified_name: Optional[str]
        """
        Verified owner's full name. Verified values are verified or provided by the payment method directly (and if supported) at the time of authorization or settlement. They cannot be set or mutated.
        """
        verified_phone: Optional[str]
        """
        Verified owner's phone number (including extension). Verified values are verified or provided by the payment method directly (and if supported) at the time of authorization or settlement. They cannot be set or mutated.
        """
        _inner_class_types = {
            "address": Address,
            "verified_address": VerifiedAddress,
        }

    class P24(StripeObject):
        reference: Optional[str]

    class Paypal(StripeObject):
        billing_agreement: Optional[str]
        fingerprint: Optional[str]
        payer_id: Optional[str]
        reference_id: Optional[str]
        reference_transaction_amount: Optional[str]
        reference_transaction_charged: Optional[bool]
        statement_descriptor: Optional[str]
        transaction_id: Optional[str]
        verified_email: Optional[str]

    class Receiver(StripeObject):
        address: Optional[str]
        """
        The address of the receiver source. This is the value that should be communicated to the customer to send their funds to.
        """
        amount_charged: int
        """
        The total amount that was moved to your balance. This is almost always equal to the amount charged. In rare cases when customers deposit excess funds and we are unable to refund those, those funds get moved to your balance and show up in amount_charged as well. The amount charged is expressed in the source's currency.
        """
        amount_received: int
        """
        The total amount received by the receiver source. `amount_received = amount_returned + amount_charged` should be true for consumed sources unless customers deposit excess funds. The amount received is expressed in the source's currency.
        """
        amount_returned: int
        """
        The total amount that was returned to the customer. The amount returned is expressed in the source's currency.
        """
        refund_attributes_method: str
        """
        Type of refund attribute method, one of `email`, `manual`, or `none`.
        """
        refund_attributes_status: str
        """
        Type of refund attribute status, one of `missing`, `requested`, or `available`.
        """

    class Redaction(StripeObject):
        status: Literal["processing", "redacted", "validated"]
        """
        Indicates whether this object and its related objects have been redacted or not.
        """

    class Redirect(StripeObject):
        failure_reason: Optional[str]
        """
        The failure reason for the redirect, either `user_abort` (the customer aborted or dropped out of the redirect flow), `declined` (the authentication failed or the transaction was declined), or `processing_error` (the redirect failed due to a technical error). Present only if the redirect status is `failed`.
        """
        return_url: str
        """
        The URL you provide to redirect the customer to after they authenticated their payment.
        """
        status: str
        """
        The status of the redirect, either `pending` (ready to be used by your customer to authenticate the transaction), `succeeded` (successful authentication, cannot be reused) or `not_required` (redirect should not be used) or `failed` (failed authentication, cannot be reused).
        """
        url: str
        """
        The URL provided to you to redirect a customer to as part of a `redirect` authentication flow.
        """

    class SepaCreditTransfer(StripeObject):
        bank_name: Optional[str]
        bic: Optional[str]
        iban: Optional[str]
        refund_account_holder_address_city: Optional[str]
        refund_account_holder_address_country: Optional[str]
        refund_account_holder_address_line1: Optional[str]
        refund_account_holder_address_line2: Optional[str]
        refund_account_holder_address_postal_code: Optional[str]
        refund_account_holder_address_state: Optional[str]
        refund_account_holder_name: Optional[str]
        refund_iban: Optional[str]

    class SepaDebit(StripeObject):
        bank_code: Optional[str]
        branch_code: Optional[str]
        country: Optional[str]
        fingerprint: Optional[str]
        last4: Optional[str]
        mandate_reference: Optional[str]
        mandate_url: Optional[str]

    class Sofort(StripeObject):
        bank_code: Optional[str]
        bank_name: Optional[str]
        bic: Optional[str]
        country: Optional[str]
        iban_last4: Optional[str]
        preferred_language: Optional[str]
        statement_descriptor: Optional[str]

    class SourceOrder(StripeObject):
        class Item(StripeObject):
            amount: Optional[int]
            """
            The amount (price) for this order item.
            """
            currency: Optional[str]
            """
            This currency of this order item. Required when `amount` is present.
            """
            description: Optional[str]
            """
            Human-readable description for this order item.
            """
            parent: Optional[str]
            """
            The ID of the associated object for this line item. Expandable if not null (e.g., expandable to a SKU).
            """
            quantity: Optional[int]
            """
            The quantity of this order item. When type is `sku`, this is the number of instances of the SKU to be ordered.
            """
            type: Optional[str]
            """
            The type of this order item. Must be `sku`, `tax`, or `shipping`.
            """

        class Shipping(StripeObject):
            class Address(StripeObject):
                city: Optional[str]
                """
                City, district, suburb, town, or village.
                """
                country: Optional[str]
                """
                Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
                """
                line1: Optional[str]
                """
                Address line 1, such as the street, PO Box, or company name.
                """
                line2: Optional[str]
                """
                Address line 2, such as the apartment, suite, unit, or building.
                """
                postal_code: Optional[str]
                """
                ZIP or postal code.
                """
                state: Optional[str]
                """
                State, county, province, or region ([ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2)).
                """

            address: Optional[Address]
            carrier: Optional[str]
            """
            The delivery service that shipped a physical product, such as Fedex, UPS, USPS, etc.
            """
            name: Optional[str]
            """
            Recipient name.
            """
            phone: Optional[str]
            """
            Recipient phone (including extension).
            """
            tracking_number: Optional[str]
            """
            The tracking number for a physical product, obtained from the delivery service. If multiple tracking numbers were generated for this purchase, please separate them with commas.
            """
            _inner_class_types = {"address": Address}

        amount: int
        """
        A positive integer in the smallest currency unit (that is, 100 cents for $1.00, or 1 for ¥1, Japanese Yen being a zero-decimal currency) representing the total amount for the order.
        """
        currency: str
        """
        Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
        """
        email: Optional[str]
        """
        The email address of the customer placing the order.
        """
        items: Optional[List[Item]]
        """
        List of items constituting the order.
        """
        shipping: Optional[Shipping]
        _inner_class_types = {"items": Item, "shipping": Shipping}

    class ThreeDSecure(StripeObject):
        address_line1_check: Optional[str]
        address_zip_check: Optional[str]
        authenticated: Optional[bool]
        brand: Optional[str]
        brand_product: Optional[str]
        card: Optional[str]
        country: Optional[str]
        customer: Optional[str]
        cvc_check: Optional[str]
        description: Optional[str]
        dynamic_last4: Optional[str]
        exp_month: Optional[int]
        exp_year: Optional[int]
        fingerprint: Optional[str]
        funding: Optional[str]
        iin: Optional[str]
        issuer: Optional[str]
        last4: Optional[str]
        name: Optional[str]
        three_d_secure: Optional[str]
        tokenization_method: Optional[str]

    class Wechat(StripeObject):
        prepay_id: Optional[str]
        qr_code_url: Optional[str]
        statement_descriptor: Optional[str]

    ach_credit_transfer: Optional[AchCreditTransfer]
    ach_debit: Optional[AchDebit]
    acss_debit: Optional[AcssDebit]
    alipay: Optional[Alipay]
    allow_redisplay: Optional[Literal["always", "limited", "unspecified"]]
    """
    This field indicates whether this payment method can be shown again to its customer in a checkout flow. Stripe products such as Checkout and Elements use this field to determine whether a payment method can be shown as a saved payment method in a checkout flow. The field defaults to “unspecified”.
    """
    amount: Optional[int]
    """
    A positive integer in the smallest currency unit (that is, 100 cents for $1.00, or 1 for ¥1, Japanese Yen being a zero-decimal currency) representing the total amount associated with the source. This is the amount for which the source will be chargeable once ready. Required for `single_use` sources.
    """
    au_becs_debit: Optional[AuBecsDebit]
    bancontact: Optional[Bancontact]
    card: Optional[Card]
    card_present: Optional[CardPresent]
    client_secret: str
    """
    The client secret of the source. Used for client-side retrieval using a publishable key.
    """
    code_verification: Optional[CodeVerification]
    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    currency: Optional[str]
    """
    Three-letter [ISO code for the currency](https://stripe.com/docs/currencies) associated with the source. This is the currency for which the source will be chargeable once ready. Required for `single_use` sources.
    """
    customer: Optional[str]
    """
    The ID of the customer to which this source is attached. This will not be present when the source has not been attached to a customer.
    """
    eps: Optional[Eps]
    flow: str
    """
    The authentication `flow` of the source. `flow` is one of `redirect`, `receiver`, `code_verification`, `none`.
    """
    giropay: Optional[Giropay]
    id: str
    """
    Unique identifier for the object.
    """
    ideal: Optional[Ideal]
    klarna: Optional[Klarna]
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    metadata: Optional[UntypedStripeObject[str]]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    multibanco: Optional[Multibanco]
    object: Literal["source"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    owner: Optional[Owner]
    """
    Information about the owner of the payment instrument that may be used or required by particular source types.
    """
    p24: Optional[P24]
    paypal: Optional[Paypal]
    receiver: Optional[Receiver]
    redaction: Optional[Redaction]
    """
    Redaction status of this source. If not null, this source is associated to a redaction job.
    """
    redirect: Optional[Redirect]
    sepa_credit_transfer: Optional[SepaCreditTransfer]
    sepa_debit: Optional[SepaDebit]
    sofort: Optional[Sofort]
    source_order: Optional[SourceOrder]
    statement_descriptor: Optional[str]
    """
    Extra information about a source. This will appear on your customer's statement every time you charge the source.
    """
    status: str
    """
    The status of the source, one of `canceled`, `chargeable`, `consumed`, `failed`, or `pending`. Only `chargeable` sources can be used to create a charge.
    """
    three_d_secure: Optional[ThreeDSecure]
    type: Literal[
        "ach_credit_transfer",
        "ach_debit",
        "acss_debit",
        "alipay",
        "au_becs_debit",
        "bancontact",
        "card",
        "card_present",
        "eps",
        "giropay",
        "ideal",
        "klarna",
        "multibanco",
        "p24",
        "paypal",
        "sepa_credit_transfer",
        "sepa_debit",
        "sofort",
        "three_d_secure",
        "wechat",
    ]
    """
    The `type` of the source. The `type` is a payment method, one of `ach_credit_transfer`, `ach_debit`, `alipay`, `bancontact`, `card`, `card_present`, `eps`, `giropay`, `ideal`, `multibanco`, `klarna`, `p24`, `sepa_debit`, `sofort`, `three_d_secure`, or `wechat`. An additional hash is included on the source with a name matching this value. It contains additional information specific to the [payment method](https://docs.stripe.com/sources) used.
    """
    usage: Optional[str]
    """
    Either `reusable` or `single_use`. Whether this source should be reusable or not. Some source types may or may not be reusable by construction, while others may leave the option at creation. If an incompatible value is passed, an error will be returned.
    """
    wechat: Optional[Wechat]

    @classmethod
    def create(cls, **params: Unpack["SourceCreateParams"]) -> "Source":
        """
        Creates a new source object.
        """
        return cast(
            "Source",
            cls._static_request(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    async def create_async(
        cls, **params: Unpack["SourceCreateParams"]
    ) -> "Source":
        """
        Creates a new source object.
        """
        return cast(
            "Source",
            await cls._static_request_async(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    def _cls_list_source_transactions(
        cls,
        source: str,
        **params: Unpack["SourceListSourceTransactionsParams"],
    ) -> ListObject["SourceTransaction"]:
        """
        List source transactions for a given source.
        """
        return cast(
            ListObject["SourceTransaction"],
            cls._static_request(
                "get",
                "/v1/sources/{source}/source_transactions".format(
                    source=sanitize_id(source)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    def list_source_transactions(
        source: str, **params: Unpack["SourceListSourceTransactionsParams"]
    ) -> ListObject["SourceTransaction"]:
        """
        List source transactions for a given source.
        """
        ...

    @overload
    def list_source_transactions(
        self, **params: Unpack["SourceListSourceTransactionsParams"]
    ) -> ListObject["SourceTransaction"]:
        """
        List source transactions for a given source.
        """
        ...

    @class_method_variant("_cls_list_source_transactions")
    def list_source_transactions(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["SourceListSourceTransactionsParams"]
    ) -> ListObject["SourceTransaction"]:
        """
        List source transactions for a given source.
        """
        return cast(
            ListObject["SourceTransaction"],
            self._request(
                "get",
                "/v1/sources/{source}/source_transactions".format(
                    source=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    async def _cls_list_source_transactions_async(
        cls,
        source: str,
        **params: Unpack["SourceListSourceTransactionsParams"],
    ) -> ListObject["SourceTransaction"]:
        """
        List source transactions for a given source.
        """
        return cast(
            ListObject["SourceTransaction"],
            await cls._static_request_async(
                "get",
                "/v1/sources/{source}/source_transactions".format(
                    source=sanitize_id(source)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    async def list_source_transactions_async(
        source: str, **params: Unpack["SourceListSourceTransactionsParams"]
    ) -> ListObject["SourceTransaction"]:
        """
        List source transactions for a given source.
        """
        ...

    @overload
    async def list_source_transactions_async(
        self, **params: Unpack["SourceListSourceTransactionsParams"]
    ) -> ListObject["SourceTransaction"]:
        """
        List source transactions for a given source.
        """
        ...

    @class_method_variant("_cls_list_source_transactions_async")
    async def list_source_transactions_async(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["SourceListSourceTransactionsParams"]
    ) -> ListObject["SourceTransaction"]:
        """
        List source transactions for a given source.
        """
        return cast(
            ListObject["SourceTransaction"],
            await self._request_async(
                "get",
                "/v1/sources/{source}/source_transactions".format(
                    source=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    def modify(
        cls, id: str, **params: Unpack["SourceModifyParams"]
    ) -> "Source":
        """
        Updates the specified source by setting the values of the parameters passed. Any parameters not provided will be left unchanged.

        This request accepts the metadata and owner as arguments. It is also possible to update type specific information for selected payment methods. Please refer to our [payment method guides](https://docs.stripe.com/docs/sources) for more detail.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "Source",
            cls._static_request(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    async def modify_async(
        cls, id: str, **params: Unpack["SourceModifyParams"]
    ) -> "Source":
        """
        Updates the specified source by setting the values of the parameters passed. Any parameters not provided will be left unchanged.

        This request accepts the metadata and owner as arguments. It is also possible to update type specific information for selected payment methods. Please refer to our [payment method guides](https://docs.stripe.com/docs/sources) for more detail.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "Source",
            await cls._static_request_async(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["SourceRetrieveParams"]
    ) -> "Source":
        """
        Retrieves an existing source object. Supply the unique source ID from a source creation request and Stripe will return the corresponding up-to-date source object information.
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["SourceRetrieveParams"]
    ) -> "Source":
        """
        Retrieves an existing source object. Supply the unique source ID from a source creation request and Stripe will return the corresponding up-to-date source object information.
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    @classmethod
    def _cls_verify(
        cls, source: str, **params: Unpack["SourceVerifyParams"]
    ) -> "Source":
        """
        Verify a given source.
        """
        return cast(
            "Source",
            cls._static_request(
                "post",
                "/v1/sources/{source}/verify".format(
                    source=sanitize_id(source)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    def verify(
        source: str, **params: Unpack["SourceVerifyParams"]
    ) -> "Source":
        """
        Verify a given source.
        """
        ...

    @overload
    def verify(self, **params: Unpack["SourceVerifyParams"]) -> "Source":
        """
        Verify a given source.
        """
        ...

    @class_method_variant("_cls_verify")
    def verify(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["SourceVerifyParams"]
    ) -> "Source":
        """
        Verify a given source.
        """
        return cast(
            "Source",
            self._request(
                "post",
                "/v1/sources/{source}/verify".format(
                    source=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    async def _cls_verify_async(
        cls, source: str, **params: Unpack["SourceVerifyParams"]
    ) -> "Source":
        """
        Verify a given source.
        """
        return cast(
            "Source",
            await cls._static_request_async(
                "post",
                "/v1/sources/{source}/verify".format(
                    source=sanitize_id(source)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    async def verify_async(
        source: str, **params: Unpack["SourceVerifyParams"]
    ) -> "Source":
        """
        Verify a given source.
        """
        ...

    @overload
    async def verify_async(
        self, **params: Unpack["SourceVerifyParams"]
    ) -> "Source":
        """
        Verify a given source.
        """
        ...

    @class_method_variant("_cls_verify_async")
    async def verify_async(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["SourceVerifyParams"]
    ) -> "Source":
        """
        Verify a given source.
        """
        return cast(
            "Source",
            await self._request_async(
                "post",
                "/v1/sources/{source}/verify".format(
                    source=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )

    def detach(self, **params) -> "Source":
        token = self.id

        if hasattr(self, "customer") and self.customer:
            extn = sanitize_id(token)
            customer = self.customer
            base = Customer.class_url()
            owner_extn = sanitize_id(customer)
            url = "%s/%s/sources/%s" % (base, owner_extn, extn)

            self._request_and_refresh("delete", url, params)
            return cast("Source", self)

        else:
            raise InvalidRequestError(
                "Source %s does not appear to be currently attached "
                "to a customer object." % token,
                "id",
            )

    _inner_class_types = {
        "ach_credit_transfer": AchCreditTransfer,
        "ach_debit": AchDebit,
        "acss_debit": AcssDebit,
        "alipay": Alipay,
        "au_becs_debit": AuBecsDebit,
        "bancontact": Bancontact,
        "card": Card,
        "card_present": CardPresent,
        "code_verification": CodeVerification,
        "eps": Eps,
        "giropay": Giropay,
        "ideal": Ideal,
        "klarna": Klarna,
        "multibanco": Multibanco,
        "owner": Owner,
        "p24": P24,
        "paypal": Paypal,
        "receiver": Receiver,
        "redaction": Redaction,
        "redirect": Redirect,
        "sepa_credit_transfer": SepaCreditTransfer,
        "sepa_debit": SepaDebit,
        "sofort": Sofort,
        "source_order": SourceOrder,
        "three_d_secure": ThreeDSecure,
        "wechat": Wechat,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing_extensions import Literal, NotRequired, TypedDict


class ImportListParams(TypedDict):
    created: NotRequired[str]
    """
    Filter for objects created at the specified timestamp.
    Must be an RFC 3339 date & time value, for example: 2022-09-18T13:22:00Z.
    """
    created_gt: NotRequired[str]
    """
    Filter for objects created after the specified timestamp.
    Must be an RFC 3339 date & time value, for example: 2022-09-18T13:22:00Z.
    """
    created_gte: NotRequired[str]
    """
    Filter for objects created on or after the specified timestamp.
    Must be an RFC 3339 date & time value, for example: 2022-09-18T13:22:00Z.
    """
    created_lt: NotRequired[str]
    """
    Filter for objects created before the specified timestamp.
    Must be an RFC 3339 date & time value, for example: 2022-09-18T13:22:00Z.
    """
    created_lte: NotRequired[str]
    """
    Filter for objects created on or before the specified timestamp.
    Must be an RFC 3339 date & time value, for example: 2022-09-18T13:22:00Z.
    """
    feed_type: NotRequired[
        Literal["inventory", "pricing", "product", "promotion"]
    ]
    """
    Filter by the type of feed data being imported.
    """
    limit: NotRequired[int]
    """
    The maximum number of results per page.
    """
    status: NotRequired[
        Literal[
            "awaiting_upload",
            "failed",
            "processing",
            "succeeded",
            "succeeded_with_errors",
        ]
    ]
    """
    Filter by import status.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.apps._secret_service import SecretService

_subservices = {"secrets": ["stripe.apps._secret_service", "SecretService"]}


class AppsService(StripeService):
    secrets: "SecretService"

    def __init__(self, requestor):
        super().__init__(requestor)

    def __getattr__(self, name):
        try:
            import_from, service = _subservices[name]
            service_class = getattr(
                import_module(import_from),
                service,
            )
            setattr(
                self,
                name,
                service_class(self._requestor),
            )
            return getattr(self, name)
        except KeyError:
            raise AttributeError()



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._expandable_field import ExpandableField
from stripe._list_object import ListObject
from stripe._listable_api_resource import ListableAPIResource
from stripe._stripe_object import StripeObject, UntypedStripeObject
from typing import ClassVar, List, Optional
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._charge import Charge
    from stripe.params.reserve._hold_list_params import HoldListParams
    from stripe.params.reserve._hold_retrieve_params import HoldRetrieveParams
    from stripe.reserve._plan import Plan


class Hold(ListableAPIResource["Hold"]):
    """
    ReserveHolds are used to place a temporary ReserveHold on a merchant's funds.
    """

    OBJECT_NAME: ClassVar[Literal["reserve.hold"]] = "reserve.hold"

    class ReleaseDetail(StripeObject):
        amount: int
        """
        The amount released by the ReserveRelease from this ReserveHold. A positive integer representing how much is released in the [smallest currency unit](https://docs.stripe.com/currencies#zero-decimal).
        """
        reserve_release: str
        """
        The ReserveRelease which released funds from this ReserveHold (e.g., resrel_123).
        """

    class ReleaseSchedule(StripeObject):
        release_after: Optional[int]
        """
        The time after which the ReserveHold is requested to be released.
        """
        scheduled_release: Optional[int]
        """
        The time at which the ReserveHold is scheduled to be released, automatically set to midnight UTC of the day after `release_after`.
        """

    amount: int
    """
    Amount reserved. A positive integer representing how much is reserved in the [smallest currency unit](https://docs.stripe.com/currencies#zero-decimal).
    """
    amount_releasable: Optional[int]
    """
    Amount in cents that can be released from this ReserveHold
    """
    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    created_by: Literal["application", "stripe"]
    """
    Indicates which party created this ReserveHold.
    """
    currency: str
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    id: str
    """
    Unique identifier for the object.
    """
    is_releasable: Optional[bool]
    """
    Whether there are any funds available to release on this ReserveHold. Note that if the ReserveHold is in the process of being released, this could be false, even though the funds haven't been fully released yet.
    """
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    metadata: Optional[UntypedStripeObject[str]]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    object: Literal["reserve.hold"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    reason: Literal["charge", "standalone"]
    """
    The reason for the ReserveHold.
    """
    release_details: Optional[List[ReleaseDetail]]
    """
    List of ReserveReleases and the amounts released from this ReserveHold.
    """
    release_schedule: ReleaseSchedule
    reserve_plan: Optional[ExpandableField["Plan"]]
    """
    The ReservePlan which produced this ReserveHold (i.e., resplan_123)
    """
    source_charge: Optional[ExpandableField["Charge"]]
    """
    The Charge which funded this ReserveHold (e.g., ch_123)
    """
    source_type: Literal["bank_account", "card", "fpx"]
    """
    Which source balance type this ReserveHold reserves funds from. One of `bank_account`, `card`, or `fpx`.
    """

    @classmethod
    def list(cls, **params: Unpack["HoldListParams"]) -> ListObject["Hold"]:
        """
        Returns a list of ReserveHolds previously created. The ReserveHolds are returned in sorted order, with the most recent ReserveHolds appearing first.
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    async def list_async(
        cls, **params: Unpack["HoldListParams"]
    ) -> ListObject["Hold"]:
        """
        Returns a list of ReserveHolds previously created. The ReserveHolds are returned in sorted order, with the most recent ReserveHolds appearing first.
        """
        result = await cls._static_request_async(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["HoldRetrieveParams"]
    ) -> "Hold":
        """
        Retrieve a ReserveHold.
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["HoldRetrieveParams"]
    ) -> "Hold":
        """
        Retrieve a ReserveHold.
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    _inner_class_types = {
        "release_details": ReleaseDetail,
        "release_schedule": ReleaseSchedule,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from stripe._stripe_object import UntypedStripeObject
from typing import Dict, List
from typing_extensions import Literal, NotRequired, TypedDict


class CustomerModifySourceParams(RequestOptions):
    account_holder_name: NotRequired[str]
    """
    The name of the person or business that owns the bank account.
    """
    account_holder_type: NotRequired[Literal["company", "individual"]]
    """
    The type of entity that holds the account. This can be either `individual` or `company`.
    """
    address_city: NotRequired[str]
    """
    City/District/Suburb/Town/Village.
    """
    address_country: NotRequired[str]
    """
    Billing address country, if provided when creating card.
    """
    address_line1: NotRequired[str]
    """
    Address line 1 (Street address/PO Box/Company name).
    """
    address_line2: NotRequired[str]
    """
    Address line 2 (Apartment/Suite/Unit/Building).
    """
    address_state: NotRequired[str]
    """
    State/County/Province/Region.
    """
    address_zip: NotRequired[str]
    """
    ZIP or postal code.
    """
    exp_month: NotRequired[str]
    """
    Two digit number representing the card's expiration month.
    """
    exp_year: NotRequired[str]
    """
    Four digit number representing the card's expiration year.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    metadata: NotRequired[
        "Literal['']|Dict[str, str]|UntypedStripeObject[str]"
    ]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    name: NotRequired[str]
    """
    Cardholder name.
    """
    owner: NotRequired["CustomerModifySourceParamsOwner"]


class CustomerModifySourceParamsOwner(TypedDict):
    address: NotRequired["CustomerModifySourceParamsOwnerAddress"]
    """
    Owner's address.
    """
    email: NotRequired[str]
    """
    Owner's email address.
    """
    name: NotRequired[str]
    """
    Owner's full name.
    """
    phone: NotRequired[str]
    """
    Owner's phone number.
    """


class CustomerModifySourceParamsOwnerAddress(TypedDict):
    city: NotRequired[str]
    """
    City, district, suburb, town, or village.
    """
    country: NotRequired[str]
    """
    Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
    """
    line1: NotRequired[str]
    """
    Address line 1, such as the street, PO Box, or company name.
    """
    line2: NotRequired[str]
    """
    Address line 2, such as the apartment, suite, unit, or building.
    """
    postal_code: NotRequired[str]
    """
    ZIP or postal code.
    """
    state: NotRequired[str]
    """
    State, county, province, or region ([ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2)).
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.params.v2.money_management._received_credit_list_params import (
        ReceivedCreditListParams,
    )
    from stripe.params.v2.money_management._received_credit_retrieve_params import (
        ReceivedCreditRetrieveParams,
    )
    from stripe.v2._list_object import ListObject
    from stripe.v2.money_management._received_credit import ReceivedCredit


class ReceivedCreditService(StripeService):
    def list(
        self,
        params: Optional["ReceivedCreditListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[ReceivedCredit]":
        """
        Retrieves a list of ReceivedCredits.
        """
        return cast(
            "ListObject[ReceivedCredit]",
            self._request(
                "get",
                "/v2/money_management/received_credits",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["ReceivedCreditListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[ReceivedCredit]":
        """
        Retrieves a list of ReceivedCredits.
        """
        return cast(
            "ListObject[ReceivedCredit]",
            await self._request_async(
                "get",
                "/v2/money_management/received_credits",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: Optional["ReceivedCreditRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ReceivedCredit":
        """
        Retrieve a ReceivedCredit by ID.
        """
        return cast(
            "ReceivedCredit",
            self._request(
                "get",
                "/v2/money_management/received_credits/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: Optional["ReceivedCreditRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ReceivedCredit":
        """
        Retrieve a ReceivedCredit by ID.
        """
        return cast(
            "ReceivedCredit",
            await self._request_async(
                "get",
                "/v2/money_management/received_credits/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject
from typing import ClassVar, List, Optional
from typing_extensions import Literal


class ConnectionSession(StripeObject):
    """
    A short-lived, single-use session used to launch client-side Link onboarding
    that connects an Account to a Link consumer account.
    """

    OBJECT_NAME: ClassVar[Literal["v2.core.connection_session"]] = (
        "v2.core.connection_session"
    )

    class Connection(StripeObject):
        granted_access: Optional[List[Literal["payout_methods"]]]
        """
        The access granted to the Account by the Connection.
        """
        type: Literal["link"]
        """
        The type of the Connection.
        """

    account: str
    """
    The Account this Connection Session was created for.
    """
    allowed_connection_types: Optional[List[Literal["link"]]]
    """
    The Connection types that the Connection Session is allowed to establish.
    """
    client_secret: str
    """
    The client secret of this Connection Session. Used on the client to set up secure access to the given Account.
    """
    connection: Optional[Connection]
    """
    The Connection created by the ConnectionSession.
    """
    created: str
    """
    Time at which the ConnectionSession was created.
    """
    id: str
    """
    The unique identifier for this ConnectionSession.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    object: Literal["v2.core.connection_session"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    requested_access: Optional[List[Literal["payout_methods"]]]
    """
    The access that is collected with the Connection Session.
    """
    _inner_class_types = {"connection": Connection}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import UntypedStripeObject
from typing import Dict, Optional
from typing_extensions import NotRequired, TypedDict


class RateCardUpdateParams(TypedDict):
    active: NotRequired[bool]
    """
    Sets whether the RateCard is active. Inactive RateCards cannot be used in new activations or have new rates added.
    """
    display_name: NotRequired[str]
    """
    A customer-facing name for the RateCard.
    This name is used in Stripe-hosted products like the Customer Portal and Checkout. It does not show up on Invoices.
    Maximum length of 250 characters.
    """
    live_version: NotRequired[str]
    """
    Changes the version that new RateCard activations use. Providing `live_version = "latest"` sets the
    RateCard's `live_version` to its latest version.
    """
    lookup_key: NotRequired[str]
    """
    An internal key you can use to search for a particular RateCard. Maximum length of 200 characters.
    """
    metadata: NotRequired[
        "Dict[str, Optional[str]]|UntypedStripeObject[Optional[str]]"
    ]
    """
    Set of [key-value pairs](https://docs.stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.v2.payments import (
        settlement_allocation_intents as settlement_allocation_intents,
    )
    from stripe.v2.payments._off_session_payment import (
        OffSessionPayment as OffSessionPayment,
    )
    from stripe.v2.payments._off_session_payment_service import (
        OffSessionPaymentService as OffSessionPaymentService,
    )
    from stripe.v2.payments._settlement_allocation_intent import (
        SettlementAllocationIntent as SettlementAllocationIntent,
    )
    from stripe.v2.payments._settlement_allocation_intent_service import (
        SettlementAllocationIntentService as SettlementAllocationIntentService,
    )
    from stripe.v2.payments._settlement_allocation_intent_split import (
        SettlementAllocationIntentSplit as SettlementAllocationIntentSplit,
    )

# name -> (import_target, is_submodule)
_import_map = {
    "settlement_allocation_intents": (
        "stripe.v2.payments.settlement_allocation_intents",
        True,
    ),
    "OffSessionPayment": ("stripe.v2.payments._off_session_payment", False),
    "OffSessionPaymentService": (
        "stripe.v2.payments._off_session_payment_service",
        False,
    ),
    "SettlementAllocationIntent": (
        "stripe.v2.payments._settlement_allocation_intent",
        False,
    ),
    "SettlementAllocationIntentService": (
        "stripe.v2.payments._settlement_allocation_intent_service",
        False,
    ),
    "SettlementAllocationIntentSplit": (
        "stripe.v2.payments._settlement_allocation_intent_split",
        False,
    ),
}
if not TYPE_CHECKING:

    def __getattr__(name):
        try:
            target, is_submodule = _import_map[name]
            module = import_module(target)
            if is_submodule:
                return module

            return getattr(
                module,
                name,
            )
        except KeyError:
            raise AttributeError()



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from stripe._stripe_object import UntypedStripeObject
from typing import Dict, List
from typing_extensions import Literal, NotRequired


class FileLinkCreateParams(RequestOptions):
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    expires_at: NotRequired[int]
    """
    The link isn't usable after this future timestamp.
    """
    file: str
    """
    The ID of the file. The file's `purpose` must be one of the following: `business_icon`, `business_logo`, `customer_signature`, `dispute_evidence`, `finance_report_run`, `financial_account_statement`, `identity_document_downloadable`, `issuing_regulatory_reporting`, `pci_document`, `selfie`, `sigma_scheduled_query`, `tax_document_user_upload`, `terminal_android_apk`, or `terminal_reader_splashscreen`.
    """
    metadata: NotRequired[
        "Literal['']|Dict[str, str]|UntypedStripeObject[str]"
    ]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing_extensions import Literal, NotRequired, TypedDict


class MoneyManagementRecipientVerificationsParams(TypedDict):
    match_result: Literal["close_match", "match", "no_match", "unavailable"]
    """
    Expected match level of the RecipientVerification to be created: `match`, `close_match`, `no_match`, `unavailable`.
    For `close_match`, the simulated response appends "close_match" to the provided name in match_result_details.matched_name.
    """
    payout_method: str
    """
    ID of the payout method.
    """
    recipient: NotRequired[str]
    """
    ID of the recipient account. Required if the recipient distinct from the sender. Leave empty if the recipient and sender are the same entity (i.e. for me-to-me payouts).
    """
